@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2019 Maxime DOYEN
+ *  Copyright (C) 1995-2020 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -114,7 +114,7 @@ gint count, i;
 		// L[Transfer account name]
 		// LCategory of transaction/Class of transaction
 		// L[Transfer account]/Class of transaction
-		if( txn->paymode == PAYMODE_INTXFER && txn->kacc == acc->key)
+		if( (txn->flags & OF_INTXFER) && (txn->kacc == acc->key) )
 		{
 		//#579260
 			Account *dstacc = da_acc_get(txn->kxferacc);
@@ -708,6 +708,12 @@ gint i, col;
 		else
 		if(txn->status == TXN_STATUS_RECONCILED)
 			ppc.column_txt[i] = "R";
+		//else
+		//if(txn->status == TXN_STATUS_REMIND)
+		//	ppc.column_txt[i] = "?";
+		else
+		if(txn->status == TXN_STATUS_VOID)
+			ppc.column_txt[i] = "v";
 		
 		i = 6;
 		hb_strfnum(balbuf, G_ASCII_DTOSTR_BUF_SIZE-1, txn->balance, txn->kcur, GLOBALS->minor);
