@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2019 Maxime DOYEN
+ *  Copyright (C) 1995-2020 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -57,6 +57,56 @@ Filter *da_flt_malloc(void)
 	return g_malloc0(sizeof(Filter));
 }
 
+void da_flt_status_cat_set(Filter *flt, guint32 kcat, gboolean status)
+{
+Category *cat = da_cat_get(kcat);
+	if(cat)
+		cat->flt_select = status;
+}
+
+
+void da_flt_status_pay_set(Filter *flt, guint32 kpay, gboolean status)
+{
+Payee *pay = da_pay_get(kpay);
+	if(pay)
+		pay->flt_select = status;
+}
+
+
+void da_flt_status_acc_set(Filter *flt, guint32 kacc, gboolean status)
+{
+Account *acc = da_acc_get(kacc);
+	if(acc)
+		acc->flt_select = status;
+}
+
+
+gboolean da_flt_status_cat_get(Filter *flt, guint32 kcat)
+{
+Category *cat = da_cat_get(kcat);
+	if(cat)
+		return cat->flt_select;
+	return FALSE;
+}
+
+
+gboolean da_flt_status_pay_get(Filter *flt, guint32 kpay)
+{
+Payee *pay = da_pay_get(kpay);
+	if(pay)
+		return pay->flt_select;
+	return FALSE;
+}
+
+
+gboolean da_flt_status_acc_get(Filter *flt, guint32 kacc)
+{
+Account *acc = da_acc_get(kacc);
+	if(acc)
+		return acc->flt_select;
+	return FALSE;
+}
+
 
 /* = = = = = = = = = = = = = = = = = = = = */
 
@@ -150,6 +200,7 @@ gint i;
 	flt->type   = FLT_TYPE_ALL;
 	flt->status = FLT_STATUS_ALL;
 	flt->forceremind = PREFS->showremind;
+	flt->forcevoid = PREFS->showvoid;
 
 	*flt->last_tab = '\0';
 }
@@ -388,8 +439,9 @@ void filter_preset_type_set(Filter *flt, gint type)
 	DB( g_print("\n[filter] preset type set\n") );
 	
 	/* any type */
+	flt->option[FILTER_TYPE] = 1;
 	flt->type = type;
-	flt->option[FILTER_AMOUNT] = 0;
+	/*flt->option[FILTER_AMOUNT] = 0;
 	flt->minamount = G_MINDOUBLE;
 	flt->maxamount = G_MINDOUBLE;
 
@@ -406,7 +458,7 @@ void filter_preset_type_set(Filter *flt, gint type)
 			flt->minamount = G_MINDOUBLE;
 			flt->maxamount = G_MAXDOUBLE;
 			break;
-	}
+	}*/
 
 }
 
@@ -417,12 +469,11 @@ void filter_preset_status_set(Filter *flt, gint status)
 	DB( g_print("\n[filter] preset status set\n") );
 	
 	/* any status */
-	flt->status = status;
+	flt->option[FILTER_TYPE] = 0;
 	flt->option[FILTER_STATUS] = 0;
 	flt->option[FILTER_CATEGORY] = 0;
-	flt->option[FILTER_PAYMODE] = 0;
-	flt->reconciled = TRUE;
-	flt->cleared  = TRUE;
+	flt->type   = FLT_TYPE_ALL;
+	flt->status = FLT_STATUS_ALL;
 	//#1602835 fautly set
 	//flt->forceadd = TRUE;
 	//flt->forcechg = TRUE;
@@ -430,36 +481,32 @@ void filter_preset_status_set(Filter *flt, gint status)
 	switch( status )
 	{
 		case FLT_STATUS_UNCATEGORIZED:
+			flt->option[FILTER_TYPE] = 2;
+			flt->type = FLT_TYPE_INTXFER;
 			flt->option[FILTER_CATEGORY] = 1;
 			filter_status_cat_clear_except(flt, 0);
-			flt->option[FILTER_PAYMODE] = 1;
-			flt->paymode[PAYMODE_INTXFER] = FALSE;
 			break;
 
 		case FLT_STATUS_UNRECONCILED:
 			flt->option[FILTER_STATUS] = 2;
-			flt->reconciled = TRUE;
-			flt->cleared = FALSE;
+			flt->status = FLT_STATUS_RECONCILED;
 			break;
 
 		case FLT_STATUS_UNCLEARED:
 			flt->option[FILTER_STATUS] = 2;
-			flt->reconciled = FALSE;
-			flt->cleared = TRUE;
+			flt->status = FLT_STATUS_CLEARED;
 			break;
 
 		case FLT_STATUS_RECONCILED:
 			flt->option[FILTER_STATUS] = 1;
-			flt->reconciled = TRUE;
-			flt->cleared = FALSE;
+			flt->status = FLT_STATUS_RECONCILED;
 			break;
 
 		case FLT_STATUS_CLEARED:
 			flt->option[FILTER_STATUS] = 1;
-			flt->reconciled = FALSE;
-			flt->cleared = TRUE;
+			flt->status = FLT_STATUS_CLEARED;
 			break;
-		
+
 	}
 }
 
@@ -496,6 +543,68 @@ gchar *retval = NULL;
 }
 
 
+/* used for quicksearch text into transaction */
+gboolean filter_tpl_search_match(gchar *needle, Archive *arc)
+{
+gboolean retval = FALSE;
+Payee *payitem;
+
+	DB( g_print("\n[filter] tpl search match\n") );
+	
+	//#1668036 always try match on txn memo first
+	if(arc->memo)
+	{
+		retval |= hb_string_utf8_strstr(arc->memo, needle, FALSE);
+	}
+	if(retval) goto end;
+	
+	//#1509485
+	if(arc->flags & OF_SPLIT)
+	{
+	guint count, i;
+	Split *split;
+
+		count = da_splits_length(arc->splits);
+		for(i=0;i<count;i++)
+		{
+		gint tmpinsert = 0;
+	
+			split = da_splits_get(arc->splits, i);
+			tmpinsert = hb_string_utf8_strstr(split->memo, needle, FALSE);
+			retval |= tmpinsert;
+			if( tmpinsert )
+				break;
+		}
+	}
+	if(retval) goto end;
+	
+	if(arc->info)
+	{
+		retval |= hb_string_utf8_strstr(arc->info, needle, FALSE);
+	}
+	if(retval) goto end;
+
+	payitem = da_pay_get(arc->kpay);
+	if(payitem)
+	{
+		retval |= hb_string_utf8_strstr(payitem->name, needle, FALSE);
+	}
+	if(retval) goto end;
+
+	//#1741339 add quicksearch for amount
+	{
+	gchar formatd_buf[G_ASCII_DTOSTR_BUF_SIZE];
+	
+		hb_strfnum(formatd_buf, G_ASCII_DTOSTR_BUF_SIZE-1, arc->amount, GLOBALS->kcur, FALSE);
+		retval |= hb_string_utf8_strstr(formatd_buf, needle, FALSE);
+	}
+
+	
+end:
+	return retval;
+}
+
+
 /* used for quicksearch text into transaction */
 gboolean filter_txn_search_match(gchar *needle, Transaction *txn, gint flags)
 {
@@ -630,17 +739,6 @@ gint insert;
 
 /*** start filtering ***/
 
-	/* force display */
-	if(flt->forceadd == TRUE && (txn->flags & OF_ADDED))
-		goto end;
-
-	if(flt->forcechg == TRUE && (txn->flags & OF_CHANGED))
-		goto end;
-
-	/* force remind if not filter on status */
-	if(flt->forceremind == TRUE && (txn->status == TXN_STATUS_REMIND))
-		goto end;
-
 /* date */
 	if(flt->option[FILTER_DATE]) {
 		insert = ( (txn->date >= flt->mindate) && (txn->date <= (flt->maxdate + flt->nbdaysfuture) ) ) ? 1 : 0;
@@ -705,17 +803,58 @@ gint insert;
 	}
 	if(!insert) goto end;
 
+/* type */
+	if(flt->option[FILTER_TYPE]) {
+		switch( flt->type )
+		{
+			case FLT_TYPE_EXPENSE:
+				insert = !(txn->flags & (OF_INCOME|OF_INTXFER)) ? 1 : 0;
+				break;
+			case FLT_TYPE_INCOME:
+				insert = (txn->flags & (OF_INCOME)) ? 1 : 0;
+				break;
+			case FLT_TYPE_INTXFER:
+				insert = (txn->flags & (OF_INTXFER)) ? 1 : 0;
+				break;
+			case FLT_TYPE_ALL:
+				insert = 1;
+				break;
+		}
+		if(flt->option[FILTER_TYPE] == 2) insert ^= 1;
+	}
+	if(!insert) goto end;
+
 /* status */
 	if(flt->option[FILTER_STATUS]) {
-	gint insert1 = 0, insert2 = 0;
 
-		if(flt->reconciled)
-			insert1 = ( txn->status == TXN_STATUS_RECONCILED ) ? 1 : 0;
-		if(flt->cleared)
-			insert2 = ( txn->status == TXN_STATUS_CLEARED ) ? 1 : 0;
-
-		insert = insert1 | insert2;
-		if(flt->option[FILTER_STATUS] == 2) insert ^= 1;
+		switch( flt->status )
+		{
+			case FLT_STATUS_CLEARED:
+				insert = (txn->status == TXN_STATUS_CLEARED) ? 1 : 0;
+				break;
+			case FLT_STATUS_RECONCILED:
+				insert = (txn->status == TXN_STATUS_RECONCILED) ? 1 : 0;
+				break;
+			case FLT_STATUS_ALL:
+				insert = 1;
+				break;
+		}
+		if(flt->option[FILTER_STATUS] == 2)
+		{	
+			//#1853531 fix for more exact uncleared/unreconciled
+			switch( flt->status )
+			{
+			case FLT_STATUS_CLEARED:	//uncleared
+				insert = (txn->status == TXN_STATUS_NONE) ? 1 : 0;
+				break;
+			case FLT_STATUS_RECONCILED:	//unreconciled
+				insert = (txn->status == TXN_STATUS_NONE) || (txn->status == TXN_STATUS_CLEARED) ? 1 : 0;
+				break;
+			default:
+				insert ^= 1;
+				break;					
+			}
+		}
 	}
 	if(!insert) goto end;
 
@@ -797,9 +936,20 @@ gint insert;
 		if(flt->option[FILTER_TEXT] == 2) insert ^= 1;
 
 	}
-	if(!insert) goto end;
+	//if(!insert) goto end;
 
 end:
+	/* force display */
+	if(!insert)
+	{
+		if( ((flt->forceadd == TRUE) && (txn->flags & OF_ADDED))
+		 || ((flt->forcechg == TRUE) && (txn->flags & OF_CHANGED))
+		 || ((flt->forceremind == TRUE) && (txn->status == TXN_STATUS_REMIND))
+		 || ((flt->forcevoid == TRUE) && (txn->status == TXN_STATUS_VOID))
+		  )
+		insert = 1;
+	}
+
 //	DB( g_print(" %d :: %d :: %d\n", flt->mindate, txn->date, flt->maxdate) );
 //	DB( g_print(" [%d] %s => %d (%d)\n", txn->account, txn->memo, insert, count) );
 	return(insert);
