@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2019 Maxime DOYEN
+ *  Copyright (C) 1995-2020 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -136,7 +136,7 @@ guint32 oldkcur;
 	DB( g_print("\n[hbfile] replace base currency\n") );
 	
 	oldkcur = GLOBALS->kcur;
-	da_cur_remove(oldkcur);
+	da_cur_delete(oldkcur);
 	item = currency_add_from_user(curfmt);
 	GLOBALS->kcur = item->key;
 	
@@ -147,7 +147,7 @@ guint32 oldkcur;
 void hbfile_change_basecurrency(guint32 key)
 {
 GList *list;
-guint32 oldkcur;
+//guint32 oldkcur;
 	
 	// set every rate to 0
 	list = g_hash_table_get_values(GLOBALS->h_cur);
@@ -165,11 +165,12 @@ guint32 oldkcur;
 	}
 	g_list_free(list);	
 
-	oldkcur = GLOBALS->kcur;
+	//oldkcur = GLOBALS->kcur;
 	GLOBALS->kcur = key;
 
+	//#1851103: absolutely no reason to do that
 	// update account with old base currency
-	list = g_hash_table_get_values(GLOBALS->h_acc);
+	/*list = g_hash_table_get_values(GLOBALS->h_acc);
 	while (list != NULL)
 	{
 	Account *acc = list->data;
@@ -180,7 +181,7 @@ guint32 oldkcur;
 		list = g_list_next(list);
 	}
 	g_list_free(list);
-	
+	*/
 
 	GLOBALS->changes_count++;
 }
@@ -208,7 +209,9 @@ GList *list;
 		lnk_txn = g_queue_peek_head_link(acc->txn_queue);
 		while (lnk_txn != NULL)
 		{
-			list = g_list_append(list, lnk_txn->data);
+			//5.3 append is damn slow
+			//list = g_list_append(list, lnk_txn->data);
+			list = g_list_prepend(list, lnk_txn->data);
 			lnk_txn = g_list_next(lnk_txn);
 		}
 	
@@ -246,7 +249,7 @@ GQueue *txn_queue;
 			if( txn->date < minjulian ) //no need to go below mindate
 				break;
 
-			if( !(txn->status == TXN_STATUS_REMIND) 
+			if( !((txn->status == TXN_STATUS_REMIND) || (txn->status == TXN_STATUS_VOID)) 
 				&& (txn->date >= minjulian) 
 				&& (txn->date <= maxjulian)
 			  )
@@ -288,7 +291,7 @@ GList *lst_acc, *lnk_acc;
 GList *lnk_txn;
 GList *lxxx, *list;
 
-	DB( g_print("\n[hbfile] !! sanity_check !! \n") );
+	DB( g_print("\n[hbfile] !! full sanity check !! \n") );
 
 	DB( g_print(" - transaction\n") );
 	lst_acc = g_hash_table_get_values(GLOBALS->h_acc);
@@ -355,7 +358,19 @@ GList *lxxx, *list;
 		list = g_list_next(list);
 	}
 	g_list_free(lxxx);
+
 	
+	DB( g_print(" - assignments\n") );
+	lxxx = list = g_hash_table_get_values(GLOBALS->h_rul);
+	while (list != NULL)
+	{
+	Assign *item = list->data;
+
+		da_asg_consistency(item);
+		list = g_list_next(list);
+	}
+	g_list_free(lxxx);
+
 }
 
 
@@ -449,8 +464,8 @@ guint cnt, i;
 
 		if(item->key != 0)
 		{
-			g_free(item->text);
-			item->text = g_strdup_printf("assign %d", item->key);
+			g_free(item->search);
+			item->search = g_strdup_printf("assign %d", item->key);
 			GLOBALS->changes_count++;
 		}
 		list = g_list_next(list);
@@ -535,18 +550,27 @@ void hbfile_change_filepath(gchar *filepath)
 
 void hbfile_cleanup(gboolean file_clear)
 {
-Transaction *txn;
+GSList *list;
+//Transaction *txn;
 
 	DB( g_print("\n[hbfile] cleanup\n") );
 	DB( g_print("- file clear is %d\n", file_clear) );
 	
 	// Free data storage
-	txn = g_trash_stack_pop(&GLOBALS->txn_stk);
+	/*txn = g_trash_stack_pop(&GLOBALS->txn_stk);
 	while( txn != NULL )
 	{
 		da_transaction_free (txn);
 		txn = g_trash_stack_pop(&GLOBALS->txn_stk);
+	}*/
+	list = GLOBALS->deltxn_list;
+	while(list != NULL)
+	{
+		da_transaction_free (list->data);
+		list = g_slist_next(list);
 	}
+	g_slist_free(GLOBALS->deltxn_list);
+
 
 	da_transaction_destroy();
 	da_archive_destroy(GLOBALS->arc_list);
@@ -556,6 +580,7 @@ Transaction *txn;
 	da_cat_destroy();
 	da_pay_destroy();
 	da_acc_destroy();
+	da_grp_destroy();
 	da_cur_destroy();
 
 	hbfile_change_owner(NULL);
@@ -574,15 +599,20 @@ void hbfile_setup(gboolean file_clear)
 
 	// Allocate data storage
 	da_cur_new();
+	da_grp_new();
 	da_acc_new();
 	da_pay_new();
 	da_cat_new();
 	da_tag_new();
 	da_asg_new();
+	//txn queue is allocated into account
 
 	GLOBALS->h_memo = g_hash_table_new_full(g_str_hash, g_str_equal, (GDestroyNotify)g_free, NULL);
 	GLOBALS->arc_list = NULL;
-	GLOBALS->txn_stk = NULL;
+
+	//#1419304 we keep the deleted txn to a trash stack	
+	//GLOBALS->txn_stk = NULL;
+	GLOBALS->deltxn_list = NULL;
 
 	if(file_clear == TRUE)
 	{
