@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2019 Maxime DOYEN
+ *  Copyright (C) 1995-2020 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -33,6 +33,19 @@ typedef struct _generic_txn		GenTxn;
 #define DST_ACC_SKIP		100010
 
 
+/* csv format validator */
+enum
+{
+	CSV_STRING,
+	CSV_DATE,
+	CSV_INT,
+	CSV_DOUBLE
+};
+
+
+
+
+
 struct _generic_file
 {
 	guint32      key;
@@ -59,10 +72,11 @@ struct _generic_acc
 	gboolean	is_ccard;
 	gboolean	is_unamed;		//if src account has no name into file
 	gdouble		initial;
-	gint		n_txnall;		//nb of txn total
-	gint		n_txnimp;		//nb of txn to import
-	gint		n_txnbaddate;   //nb of txn with bad date
-	gint		n_txnduplicate; //nb of txn with duplicate
+	guint		n_txnall;		//nb of txn total
+	guint		n_txnimp;		//nb of txn to import
+	guint		n_txnbaddate;   //nb of txn with bad date
+	guint		n_txnsimimp;	//nb of txn similar import
+	guint		n_txnsimdst;	//nb of txn similar dstacccount
 };
 
 
