@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2019 Maxime DOYEN
+ *  Copyright (C) 1995-2020 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -41,7 +41,7 @@ extern struct Preferences *PREFS;
 
 /* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
 
-void
+static void
 da_transaction_clean(Transaction *item)
 {
 	if(item != NULL)
@@ -58,7 +58,6 @@ da_transaction_clean(Transaction *item)
 		}
 		if(item->tags != NULL)
 		{
-			DB( g_print(" -> item->tags %p\n", item->tags) );
 			g_free(item->tags);
 			item->tags = NULL;
 		}
@@ -69,12 +68,6 @@ da_transaction_clean(Transaction *item)
 			item->splits = NULL;
 			item->flags &= ~(OF_SPLIT); //Flag that Splits are cleared
 		}
-
-		if(item->same != NULL)
-		{
-			g_list_free(item->same);
-			item->same = NULL;
-		}
 	}
 }
 
@@ -97,29 +90,49 @@ da_transaction_malloc(void)
 }
 
 
+Transaction *da_transaction_init(Transaction *txn, guint32 kacc)
+{
+	DB( g_print("da_transaction_init\n") );
+
+	da_transaction_clean(txn);
+	memset(txn, 0, sizeof(Transaction));
+
+	//fix: 318733 / 1335285
+	if( PREFS->heritdate == FALSE ) 
+		txn->date = GLOBALS->today;
+
+	txn->kacc = kacc;
+	
+	da_transaction_set_default_template(txn);
+	
+	return txn;
+}
+
+
 Transaction *da_transaction_init_from_template(Transaction *txn, Archive *arc)
 {
 	DB( g_print("da_transaction_init_from_template\n") );
 
+	da_transaction_clean(txn);
+	
 	//txn->date		= 0;
 	txn->amount	= arc->amount;
 	//#1258344 keep the current account if tpl is empty
 	if(arc->kacc)
 		txn->kacc	= arc->kacc;
 	txn->paymode	= arc->paymode;
-	txn->flags		= arc->flags | OF_ADDED;
+	txn->flags		= arc->flags;
 	txn->status		= arc->status;
 	txn->kpay		= arc->kpay;
 	txn->kcat		= arc->kcat;
 	txn->kxferacc	= arc->kxferacc;
-	txn->memo	    = g_strdup(arc->memo);
-	txn->info		= NULL;
+	if(arc->memo != NULL)
+		txn->memo	    = g_strdup(arc->memo);
+	if(arc->info != NULL)
+		txn->info	    = g_strdup(arc->info);
 
-	//copy tags (with free previous here)
-	g_free(txn->tags);
 	txn->tags = tags_clone(arc->tags);
 
-	da_split_destroy (txn->splits);
 	txn->splits = da_splits_clone(arc->splits);
 	if( da_splits_length (txn->splits) > 0 )
 		txn->flags |= OF_SPLIT; //Flag that Splits are active
@@ -219,15 +232,22 @@ GList *lacc, *list;
 	Account *acc = list->data;
 
 		g_queue_foreach(acc->txn_queue, (GFunc)da_transaction_queue_free_ghfunc, NULL);
+		//txn queue is freed into account
 		list = g_list_next(list);
 	}
 	g_list_free(lacc);
 }
 
 
+// used from register only
 static gint da_transaction_compare_datafunc(Transaction *a, Transaction *b, gpointer data)
 {
-	return ((gint)a->date - b->date);
+gint retval = (gint)a->date - b->date;
+
+	if(!retval) //#1749457
+		retval = a->pos - b->pos;
+
+	return retval;
 }
 
 
@@ -335,7 +355,7 @@ guint32 max_key = 0;
 		{
 		Transaction *item = list->data;
 
-			if( item->paymode == PAYMODE_INTXFER )
+			if( item->flags & OF_INTXFER )
 			{
 				max_key = MAX(max_key, item->kxfer);
 			}
@@ -395,6 +415,9 @@ guint nbsplit;
 
 	DB( g_print("\n[transaction] consistency\n") );
 
+
+	DB( g_print(" %d %.2f %s\n", item->date, item->amount, item->memo) );
+
 	// ensure date is between range
 	item->date = CLAMP(item->date, HB_MINDATE, HB_MAXDATE);
 
@@ -438,15 +461,29 @@ guint nbsplit;
 		GLOBALS->changes_count++;
 	}
 
-	// reset dst acc for non xfer transaction
-	if( item->paymode != PAYMODE_INTXFER )
+	// 5.3: fix split on intxfer
+	if( ((item->flags & OF_INTXFER) || (item->paymode == OLDPAYMODE_INTXFER)) && (item->splits != NULL) )
 	{
+		g_warning("txn consistency: fixed invalid split on xfer");
+		item->flags &= ~(OF_INTXFER);
+		item->paymode = PAYMODE_XFER;
 		item->kxfer = 0;
 		item->kxferacc = 0;
 	}
 
-	// check dst account exists
-	if( item->paymode == PAYMODE_INTXFER )
+	// reset dst acc for non xfer transaction
+	if( !((item->flags & OF_INTXFER) || (item->paymode == OLDPAYMODE_INTXFER)) )
+	{
+		if( (item->kxfer != 0) || (item->kxferacc != 0) )
+		{
+			g_warning("txn consistency: fixed invalid xfer");
+			item->kxfer = 0;
+			item->kxferacc = 0;
+		}
+	}
+		
+	// intxfer: check dst account exists
+	if( (item->flags & OF_INTXFER) || (item->paymode == OLDPAYMODE_INTXFER) )
 	{
 	gint tak = item->kxferacc;
 
@@ -485,7 +522,7 @@ static void transaction_xfer_create_child(Transaction *ope)
 {
 Transaction *child;
 Account *acc;
-gchar swap;
+guint32 swap;
 
 	DB( g_print("\n[transaction] xfer_create_child\n") );
 
@@ -493,13 +530,13 @@ gchar swap;
 	{
 		child = da_transaction_clone(ope);
 
-		ope->flags |= OF_CHANGED;
-		child->flags |= OF_ADDED;
+		ope->flags |= (OF_CHANGED | OF_INTXFER);
+		child->flags |= (OF_ADDED | OF_INTXFER);
 
 		child->amount = -child->amount;
 		child->flags ^= (OF_INCOME);	// invert flag
 		//#1268026 #1690555
-		if( child->status != TXN_STATUS_REMIND )
+		if( (child->status == TXN_STATUS_CLEARED) || (child->status == TXN_STATUS_RECONCILED) )
 			child->status = TXN_STATUS_NONE;
 		//child->flags &= ~(OF_VALID);	// delete reconcile state
 
@@ -538,36 +575,34 @@ gchar swap;
 
 
 //todo: add strong control and extend to payee, maybe memo ?
-static gboolean transaction_xfer_child_might(Transaction *stxn, Transaction *dtxn, gint daygap)
+// #1708974 enable different date
+static gboolean transaction_xfer_child_might(Transaction *stxn, Transaction *dtxn, guint32 daygap)
 {
 gboolean retval = FALSE;
 
-	//DB( g_print("\n[transaction] xfer_child_might\n") );
+	DB( g_print("\n[transaction] xfer_child_might\n") );
 
 	if(stxn == dtxn)
 		return FALSE;
 
-	/*g_print("test\n");
-
-	g_print(" %d %d %d %f %d\n", 
-		stxn->kcur, stxn->date, stxn->kacc, ABS(stxn->amount), stxn->kxfer );
-
-
-	g_print(" %d %d %d %f %d\n", 
-		dtxn->kcur, dtxn->date, dtxn->kacc, ABS(dtxn->amount), dtxn->kxfer );
-	*/
+	DB( g_print(" %d %d %d %f %d\n", stxn->kcur, stxn->date, stxn->kacc, ABS(stxn->amount), stxn->kxfer ) );
+	DB( g_print(" %d %d %d %f %d\n", dtxn->kcur, dtxn->date, dtxn->kacc, ABS(dtxn->amount), dtxn->kxfer ) );
 
 	if( stxn->kcur == dtxn->kcur &&
-	    stxn->date == dtxn->date &&
+	    //stxn->date == dtxn->date &&
+		//# 1708974 enable different date
+		(dtxn->date <= (stxn->date + daygap)) &&
+	    (dtxn->date >= (stxn->date - daygap)) &&
 	    //v5.1 make no sense: stxn->kxferacc == dtxn->kacc &&
 	    stxn->kacc != dtxn->kacc &&
 	    ABS(stxn->amount) == ABS(dtxn->amount) &&
 	    dtxn->kxfer == 0)
 	{
+		//TODO: we could evaluate here a match rate based on several fields
 		retval = TRUE;
 	}
 
-	//g_print(" return %d\n", retval);
+	DB( g_print(" return %d\n", retval) );
 	return retval;
 }
 
@@ -576,7 +611,8 @@ static GList *transaction_xfer_child_might_list_get(Transaction *ope, guint32 kd
 {
 GList *lst_acc, *lnk_acc;
 GList *list, *matchlist = NULL;
-
+#define XFER_DAY_GAP 2
+	
 	DB( g_print("\n[transaction] xfer_child_might_list_get\n") );
 
 	DB( g_print(" - kdstacc:%d\n", kdstacc) );
@@ -594,11 +630,11 @@ GList *list, *matchlist = NULL;
 			{
 			Transaction *item = list->data;
 	
-				// no need to go higher than src txn date
-				if(item->date < ope->date)
+				// no need to go higher than src txn date - daygap
+				if(item->date < (ope->date - XFER_DAY_GAP))
 					break;
 		
-				if( transaction_xfer_child_might(ope, item, 0) == TRUE )
+				if( transaction_xfer_child_might(ope, item, XFER_DAY_GAP) == TRUE )
 				{
 					DB( g_print(" - match : %d %s %f %d=>%d\n", item->date, item->memo, item->amount, item->kacc, item->kxferacc) );
 					matchlist = g_list_append(matchlist, item);
@@ -693,10 +729,9 @@ GList *list;
 		//#1252230
 		//if( item->paymode == PAYMODE_INTXFER 
 		//	&& item->kacc == src->kxferacc
-		//	&& item->kxfer == src->kxfer )
-		if( item->paymode == PAYMODE_INTXFER 
-		 && item->kxfer == src->kxfer 
-		 && item != src )
+		if( (item->flags & OF_INTXFER) 
+		 && (item->kxfer == src->kxfer) 
+		 && (item != src) )
 		{
 			DB( g_print(" - found : %d %s %f %d=>%d - %d\n", item->date, item->memo, item->amount, item->kacc, item->kxferacc, src->kxfer) );
 			return item;
@@ -710,7 +745,7 @@ GList *list;
 
 
 
-
+//TODO: should be static but used in hb_import
 void transaction_xfer_change_to_child(Transaction *ope, Transaction *child)
 {
 Account *dstacc;
@@ -720,10 +755,8 @@ Account *dstacc;
 	if(ope->kcur != child->kcur)
 		return;
 
-	ope->flags |= OF_CHANGED;
-	child->flags |= OF_CHANGED;
-
-	child->paymode = PAYMODE_INTXFER;
+	ope->flags |= (OF_CHANGED | OF_INTXFER);
+	child->flags |= (OF_ADDED | OF_INTXFER);
 
 	ope->kxferacc = child->kacc;
 	child->kxferacc = ope->kacc;
@@ -762,7 +795,8 @@ Account *acc;
 
 	account_balances_sub (child);
 
-	child->date		= s_txn->date;
+	//# 1708974 enable different date
+	//child->date		= s_txn->date;
 	child->amount   = -s_txn->amount;
 	child->flags	= child->flags | OF_CHANGED;
 	//#1295877
@@ -819,7 +853,8 @@ Transaction *dst;
 			g_queue_remove(acc->txn_queue, dst);
 			//#1419304 we keep the deleted txn to a trash stack	
 			//da_transaction_free (dst);
-			g_trash_stack_push(&GLOBALS->txn_stk, dst);
+			//g_trash_stack_push(&GLOBALS->txn_stk, dst);
+			GLOBALS->deltxn_list = g_slist_prepend(GLOBALS->deltxn_list, dst);
 
 			//#1691992
 			acc->flags |= AF_CHANGED;
@@ -853,7 +888,8 @@ GList *list;
 			if(item->date > src->date)
 				break;
 
-			if( item->paymode == PAYMODE_INTXFER)
+			// keep this one for backward compatibility
+			if( item->paymode == OLDPAYMODE_INTXFER)
 			{
 				if( src->date == item->date &&
 					src->kacc == item->kxferacc &&
@@ -878,6 +914,19 @@ GList *list;
 /* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
 
 
+gboolean transaction_is_balanceable(Transaction *ope)
+{
+gboolean retval = TRUE;
+
+	if( (ope->status == TXN_STATUS_VOID) )
+		retval = FALSE;
+	else if( (ope->status == TXN_STATUS_REMIND) && (PREFS->includeremind == FALSE) )
+		retval = FALSE;
+
+	   return retval;
+}
+
+
 void transaction_remove(Transaction *ope)
 {
 Account *acc;
@@ -888,7 +937,7 @@ Account *acc;
 
 	account_balances_sub(ope);
 
-	if( ope->paymode == PAYMODE_INTXFER )
+	if( ope->flags & OF_INTXFER )
 	{
 		transaction_xfer_remove_child( ope );
 	}
@@ -897,7 +946,8 @@ Account *acc;
 	acc->flags |= AF_CHANGED;
 	//#1419304 we keep the deleted txn to a trash stack	
 	//da_transaction_free(entry);
-	g_trash_stack_push(&GLOBALS->txn_stk, ope);
+	//g_trash_stack_push(&GLOBALS->txn_stk, ope);
+	GLOBALS->deltxn_list = g_slist_prepend(GLOBALS->deltxn_list, ope);
 }
 
 
@@ -931,7 +981,7 @@ Account *acc;
 
 	ope->kcur = acc->kcur;
 	
-	if(ope->paymode == PAYMODE_INTXFER)
+	if(ope->flags & OF_INTXFER)
 	{
 		acc = da_acc_get(ope->kxferacc);
 		if(acc == NULL) return NULL;
@@ -990,7 +1040,7 @@ Account *acc;
 
 		account_balances_add(newope);
 
-		if(newope->paymode == PAYMODE_INTXFER)
+		if(newope->flags & OF_INTXFER)
 		{
 			transaction_xfer_search_or_add_child(parent, newope, newope->kxferacc);
 		}
@@ -1037,226 +1087,6 @@ Account *oacc, *nacc;
 /* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
 
 
-static gboolean misc_text_match(gchar *text, gchar *searchtext, gboolean exact)
-{
-gboolean match = FALSE;
-
-	if(text == NULL)
-		return FALSE;
-	
-	//DB( g_print("search %s in %s\n", rul->name, ope->memo) );
-	if( searchtext != NULL )
-	{
-		if( exact == TRUE )
-		{
-			if( g_strrstr(text, searchtext) != NULL )
-			{
-				DB( g_print("-- found case '%s'\n", searchtext) );
-				match = TRUE;
-			}
-		}
-		else
-		{
-		gchar *word   = g_utf8_casefold(text, -1);
-		gchar *needle = g_utf8_casefold(searchtext, -1);
-
-			if( g_strrstr(word, needle) != NULL )
-			{
-				DB( g_print("-- found nocase '%s'\n", searchtext) );
-				match = TRUE;
-			}
-			g_free(word);
-			g_free(needle);
-		}
-	}
-
-	return match;
-}
-
-static gboolean misc_regex_match(gchar *text, gchar *searchtext, gboolean exact)
-{
-gboolean match = FALSE;
-
-	if(text == NULL)
-		return FALSE;
-	
-	DB( g_print("-- match RE %s in %s\n", searchtext, text) );
-	if( searchtext != NULL )
-	{
-		match = g_regex_match_simple(searchtext, text, ((exact == TRUE)?0:G_REGEX_CASELESS) | G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY );
-		if (match == TRUE) { DB( g_print("-- found pattern '%s'\n", searchtext) ); }
-	}
-	return match;
-}
-
-
-static GList *transaction_auto_assign_eval_txn(GList *l_rul, Transaction *txn)
-{
-GList *ret_list = NULL;
-GList *list;
-gchar *text;
-
-	list = g_list_first(l_rul);
-	while (list != NULL)
-	{
-	Assign *rul = list->data;
-
-		text = txn->memo;
-		if(rul->field == 1) //payee
-		{
-		Payee *pay = da_pay_get(txn->kpay);
-			if(pay)
-				text = pay->name;
-		}
-		
-		if( !(rul->flags & ASGF_REGEX) )
-		{
-			if( misc_text_match(text, rul->text, rul->flags & ASGF_EXACT) )
-				ret_list = g_list_append(ret_list, rul);
-		}
-		else
-		{
-			if( misc_regex_match(text, rul->text, rul->flags & ASGF_EXACT) )
-				ret_list = g_list_append(ret_list, rul);
-		}
-
-		list = g_list_next(list);
-	}
-
-	DB( g_print("- evaluated txn '%s'=> %d match\n", text, g_list_length (ret_list)) );
-	
-	return ret_list;
-}
-
-
-static GList *transaction_auto_assign_eval(GList *l_rul, gchar *text)
-{
-GList *ret_list = NULL;
-GList *list;
-
-	list = g_list_first(l_rul);
-	while (list != NULL)
-	{
-	Assign *rul = list->data;
-
-		if( rul->field == 0 )   //memo
-		{
-			if( !(rul->flags & ASGF_REGEX) )
-			{
-				if( misc_text_match(text, rul->text, rul->flags & ASGF_EXACT) )
-					ret_list = g_list_append(ret_list, rul);
-			}
-			else
-			{
-				if( misc_regex_match(text, rul->text, rul->flags & ASGF_EXACT) )
-					ret_list = g_list_append(ret_list, rul);
-			}
-		}
-		list = g_list_next(list);
-	}
-
-	DB( g_print("- evaluated split '%s' => %d match\n", text, g_list_length (ret_list)) );
-	
-	return ret_list;
-}
-
-
-guint transaction_auto_assign(GList *ope_list, guint32 kacc)
-{
-GList *l_ope;
-GList *l_rul;
-GList *l_match, *l_tmp;
-guint changes = 0;
-
-	DB( g_print("\n[transaction] auto_assign\n") );
-
-	l_rul = g_hash_table_get_values(GLOBALS->h_rul);
-
-	l_ope = g_list_first(ope_list);
-	while (l_ope != NULL)
-	{
-	Transaction *ope = l_ope->data;
-	gboolean changed = FALSE; 
-
-		DB( g_print("\n- work on txn '%s' : acc=%d, pay=%d, cat=%d, %s\n", ope->memo, ope->kacc, ope->kpay, ope->kcat, (ope->flags & OF_SPLIT) ? "is_split" : "" ) );
-
-		//#1215521: added kacc == 0
-		if( (kacc == ope->kacc || kacc == 0) )
-		{
-			if( !(ope->flags & OF_SPLIT) )
-			{
-				l_match = l_tmp = transaction_auto_assign_eval_txn(l_rul, ope);
-				while( l_tmp != NULL )
-				{
-				Assign *rul = l_tmp->data;
-					
-					if( (ope->kpay == 0 && (rul->flags & ASGF_DOPAY)) || (rul->flags & ASGF_OVWPAY) )
-					{
-						if(ope->kpay != rul->kpay) { changed = TRUE; }
-						ope->kpay = rul->kpay;
-					}
-
-					if( (ope->kcat == 0 && (rul->flags & ASGF_DOCAT)) || (rul->flags & ASGF_OVWCAT) )
-					{
-						if(ope->kcat != rul->kcat) { changed = TRUE; }
-						ope->kcat = rul->kcat;
-					}
-
-					if( (ope->paymode == 0 && (rul->flags & ASGF_DOMOD)) || (rul->flags & ASGF_OVWMOD) )
-					{
-						//ugly hack - don't allow modify intxfer
-						if(ope->paymode != PAYMODE_INTXFER && rul->paymode != PAYMODE_INTXFER) 
-						{
-							if(ope->paymode != rul->paymode) { changed = TRUE; }
-							ope->paymode = rul->paymode;
-						}
-					}
-					l_tmp = g_list_next(l_tmp);
-				}
-				g_list_free(l_match);
-			}
-			else
-			{
-			guint i, nbsplit = da_splits_length(ope->splits);
-				
-				for(i=0;i<nbsplit;i++)
-				{
-				Split *split = da_splits_get(ope->splits, i);
-					
-					DB( g_print("- eval split '%s'\n", split->memo) );
-
-					l_match = l_tmp = transaction_auto_assign_eval(l_rul, split->memo);
-					while( l_tmp != NULL )
-					{
-					Assign *rul = l_tmp->data;
-
-						//#1501144: check if user wants to set category in rule
-						if( (split->kcat == 0 || (rul->flags & ASGF_OVWCAT)) && (rul->flags & ASGF_DOCAT) )
-						{
-							if(split->kcat != rul->kcat) { changed = TRUE; }
-							split->kcat = rul->kcat;
-						}
-						l_tmp = g_list_next(l_tmp);
-					}	
-					g_list_free(l_match);
-				}
-			}
-
-			if(changed == TRUE)
-			{
-				ope->flags |= OF_CHANGED;
-				changes++;
-			}
-		}
-
-		l_ope = g_list_next(l_ope);
-	}
-
-	g_list_free(l_rul);
-
-	return changes;
-}
-
 
 
 static gboolean transaction_similar_match(Transaction *stxn, Transaction *dtxn, guint32 daygap)
@@ -1289,7 +1119,9 @@ GList *lnk_txn;
 	while (lnk_txn != NULL)
 	{
 	Transaction *stxn = lnk_txn->data;
-		stxn->marker = TXN_MARK_NONE;
+
+		stxn->dspflags &= ~(TXN_DSPFLG_DUPSRC|TXN_DSPFLG_DUPDST);
+		//stxn->marker = TXN_MARK_NONE;
 		lnk_txn = g_list_previous(lnk_txn);
 	}
 }
@@ -1354,12 +1186,15 @@ gint nbdup = 0;
 				break;
 			}
 				
-			if( dtxn->marker == TXN_MARK_NONE )
+			//if( dtxn->marker == TXN_MARK_NONE )
+			if( (dtxn->dspflags & (TXN_DSPFLG_DUPSRC|TXN_DSPFLG_DUPDST)) == 0 )
 			{
 				if( transaction_similar_match(stxn, dtxn, daygap) )
 				{
-					stxn->marker = TXN_MARK_DUPSRC;
-					dtxn->marker = TXN_MARK_DUPDST;
+					//stxn->marker = TXN_MARK_DUPSRC;
+					stxn->dspflags |= TXN_DSPFLG_DUPSRC;
+					//dtxn->marker = TXN_MARK_DUPDST;
+					dtxn->dspflags |= TXN_DSPFLG_DUPDST;
 					DB( g_print(" = dtxn marker=%d\n", dtxn->marker) );
 					nball++;
 				}
@@ -1374,7 +1209,8 @@ gint nbdup = 0;
 		}
 	
 		DB( g_print(" = stxn marker=%d\n", stxn->marker) );
-		if( stxn->marker == TXN_MARK_DUPSRC )
+		//if( stxn->marker == TXN_MARK_DUPSRC )
+		if( stxn->dspflags & TXN_DSPFLG_DUPSRC )
 			nbdup++;
 		
 		lnk_txn = g_list_previous(lnk_txn);
