@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2019 Maxime DOYEN
+ *  Copyright (C) 1995-2020 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -25,14 +25,13 @@
 #endif
 
 
-#include <ctype.h> 		/* isprint */
 #include <errno.h>
 #include <math.h>
 #include <libintl.h>
 #include <locale.h>
 #include <stdlib.h>		/* atoi, atof, atol */
 #include <string.h>		/* memset, memcpy, strcmp, strcpy */
-#include <time.h>
+//#include <time.h>
 
 #include <glib.h>
 #include <glib/gstdio.h>
@@ -43,6 +42,7 @@
 
 #include "hb-transaction.h"
 #include "hb-currency.h"
+#include "hb-group.h"
 #include "hb-account.h"
 #include "hb-archive.h"
 #include "hb-assign.h"
@@ -74,14 +74,14 @@
 
 
 #define HB_VERSION_MAJOR	5
-#define HB_VERSION_MINOR	2
-#define HB_VERSION_MICRO	8
+#define HB_VERSION_MINOR	3
+#define HB_VERSION_MICRO	1
 
-#define HB_VERSION		"5.2.8"
+#define HB_VERSION		"5.3.1"
 #define HB_VERSION_NUM	(HB_VERSION_MAJOR*10000) + (HB_VERSION_MINOR*100) + HB_VERSION_MICRO
 
-#define FILE_VERSION		1.3
-#define PREF_VERSION		528
+#define FILE_VERSION		1.4
+#define PREF_VERSION		531
 
 #if HB_UNSTABLE == FALSE
 	#define	PROGNAME		"HomeBank"
@@ -137,6 +137,17 @@
 
 #define HB_NUMBER_SAMPLE	1234567.89
 
+/* hbfile/account/import update flags */
+enum
+{
+	UF_TITLE     	= 1 << 0,	//1
+	UF_SENSITIVE 	= 1 << 1,	//2
+	UF_BALANCE   	= 1 << 2,	//4
+	UF_VISUAL    	= 1 << 3,	//8
+	UF_REFRESHALL	= 1 << 4	//16
+};
+
+
 
 typedef enum
 {
@@ -192,8 +203,8 @@ typedef enum
 #define ICONNAME_WINDOW_CLOSE		"window-close-symbolic"
 #define ICONNAME_LIST_ADD			"list-add-symbolic"
 #define ICONNAME_LIST_EDIT			"document-edit-symbolic"
-#define ICONNAME_LIST_REMOVE		"list-remove-symbolic"
-#define ICONNAME_LIST_REMOVE_ALL	"list-remove-all-symbolic"
+#define ICONNAME_LIST_DELETE		"list-remove-symbolic"
+#define ICONNAME_LIST_DELETE_ALL	"list-remove-all-symbolic"
 #define ICONNAME_CHANGES_PREVENT	"changes-prevent-symbolic"
 #define ICONNAME_SYSTEM_SEARCH		"system-search-symbolic"
 
@@ -205,6 +216,9 @@ typedef enum
 #define ICONNAME_HB_TOGGLE_SIGN		"toggle-sign-symbolic"
 #define ICONNAME_HB_LIST_MERGE		"list-merge-symbolic"
 
+#define ICONNAME_HB_TEXT_CASE		"text-casesensitive-symbolic"
+#define ICONNAME_HB_TEXT_REGEX		"text-regularexpression-symbolic"
+
 
 /* -------- named icons (Custom to homebank) -------- */
 
@@ -256,6 +270,8 @@ typedef enum
 #define ICONNAME_HB_OPE_DELETE      "hb-ope-delete" //? "edit-delete"
 #define ICONNAME_CONVERT			"hb-ope-convert"	// obsolete ?
 #define ICONNAME_HB_ASSIGN_RUN      "hb-assign-run"
+#define ICONNAME_HB_OPE_MOVUP		"hb-go-up"
+#define ICONNAME_HB_OPE_MOVDW		"hb-go-down"
 
 #define ICONNAME_HB_OPE_NEW		     "hb-ope-new"
 // edit is defined above
@@ -267,6 +283,7 @@ typedef enum
 #define ICONNAME_HB_OPE_RECONCILED  "hb-ope-reconciled"
 #define ICONNAME_HB_OPE_FUTURE      "hb-ope-future"
 
+#define ICONNAME_HB_PM_INTXFER		"pm-intransfer"
 
 /*
 ** Global application datas
@@ -275,6 +292,7 @@ struct HomeBank
 {
 	// hbfile storage
 	GHashTable		*h_cur;			//currencies
+	GHashTable		*h_grp;			//groups
 	GHashTable		*h_acc;			//accounts
 	GHashTable		*h_pay;			//payees
 	GHashTable		*h_cat;			//categories
@@ -286,7 +304,8 @@ struct HomeBank
 	GList			*arc_list;		//archives
 
 	//#1419304 we keep the deleted txn to a stack trash
-	GTrashStack		*txn_stk;
+	//GTrashStack		*txn_stk;
+	GSList			*deltxn_list;
 
 	// hbfile (saved properties)
 	gchar			*owner;
