@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2019 Maxime DOYEN
+ *  Copyright (C) 1995-2020 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -108,7 +108,7 @@ static   gint
 			gchar *name1 = NULL;
 			gchar *name2 = NULL;
 
-				if( ope1->paymode == PAYMODE_INTXFER )
+				if( ope1->flags & OF_INTXFER )
 				{
 				Account *acc = da_acc_get(ope1->kxferacc);
 					name1 = (acc != NULL) ? acc->name : NULL;
@@ -119,7 +119,7 @@ static   gint
 					name1 = (pay != NULL) ? pay->name : NULL;
 				}
 
-				if( ope2->paymode == PAYMODE_INTXFER )
+				if( ope2->flags & OF_INTXFER )
 				{
 				Account *acc = da_acc_get(ope2->kxferacc);
 					name2 = (acc != NULL) ? acc->name : NULL;
@@ -199,7 +199,8 @@ static void list_txn_eval_future(GtkCellRenderer *renderer, Transaction *txn)
 		NULL);
 	}
 	
-	if( txn->marker == TXN_MARK_DUPDST )
+	//if( txn->marker == TXN_MARK_DUPDST )
+	if( txn->dspflags & TXN_DSPFLG_DUPDST )
 	{
 		g_object_set(renderer, 
 		//	"strikethrough-set", TRUE,
@@ -212,7 +213,8 @@ static void list_txn_eval_future(GtkCellRenderer *renderer, Transaction *txn)
 		NULL);
 	}
 
-	if( txn->marker == TXN_MARK_DUPSRC )
+	//if( txn->marker == TXN_MARK_DUPSRC )
+	if( txn->dspflags & TXN_DSPFLG_DUPSRC )
 	{
 		g_object_set(renderer, 
 		//	"weight-set", TRUE,
@@ -256,7 +258,8 @@ gchar *iconname = NULL;
 			iconname = ( txn->flags & OF_CHANGED  ) ? ICONNAME_HB_OPE_EDIT : NULL;
 			break;
 		case 3:
-			iconname = ( txn->marker == TXN_MARK_DUPDST ) ? ICONNAME_HB_OPE_SIMILAR : NULL;
+			//iconname = ( txn->marker == TXN_MARK_DUPDST ) ? ICONNAME_HB_OPE_SIMILAR : NULL;
+			iconname = ( txn->dspflags & TXN_DSPFLG_DUPDST ) ? ICONNAME_HB_OPE_SIMILAR : NULL;
 			break;
 		/*case 3:
 			if( entry->flags & OF_VALID )
@@ -331,7 +334,10 @@ Transaction *ope;
 	switch(GPOINTER_TO_INT(user_data))
 	{
 		case 1:
-			g_object_set(renderer, "icon-name", get_paymode_icon_name(ope->paymode), NULL);
+			if(ope->flags & OF_INTXFER)
+				g_object_set(renderer, "icon-name", ICONNAME_HB_PM_INTXFER, NULL);
+			else
+				g_object_set(renderer, "icon-name", get_paymode_icon_name(ope->paymode), NULL);
 			break;
 		case 2:
 			list_txn_eval_future(renderer, ope);
@@ -349,7 +355,7 @@ gchar *direction = "";
 	gtk_tree_model_get(model, iter, MODEL_TXN_POINTER, &ope, -1);
 	list_txn_eval_future(renderer, ope);
 
-	if(ope->paymode == PAYMODE_INTXFER)
+	if(ope->flags & OF_INTXFER)
 	{
 		direction = ( ope->flags & OF_INCOME ) ? "<" : ">";
 	}
@@ -368,7 +374,7 @@ Transaction *ope;
 	list_txn_eval_future(renderer, ope);
 
 	//#926782
-	if(ope->paymode == PAYMODE_INTXFER)
+	if(ope->flags & OF_INTXFER)
 	{
 	Account *acc = da_acc_get(ope->kxferacc);
 		
@@ -439,7 +445,7 @@ gchar *iconname = NULL;
 		case TXN_STATUS_CLEARED:	iconname = ICONNAME_HB_OPE_CLEARED; break;
 		case TXN_STATUS_RECONCILED: iconname = ICONNAME_HB_OPE_RECONCILED; break;
 		case TXN_STATUS_REMIND:     iconname = ICONNAME_HB_OPE_REMIND; break;
-		
+		case TXN_STATUS_VOID:     iconname = ICONNAME_HB_OPE_VOID; break;		
 	}
 
 	//g_object_set(renderer, "text", c, NULL);
@@ -492,9 +498,10 @@ gchar *color;
 	//if(amount != 0)
 	//{
 		hb_strfmon(buf, G_ASCII_DTOSTR_BUF_SIZE-1, amount, ope->kcur, GLOBALS->minor);
-
+	
 		color = get_normal_color_amount(amount);
-		if( (column == LST_DSPOPE_BALANCE) && (ope->overdraft == TRUE) && (PREFS->custom_colors == TRUE) )
+		//if( (column == LST_DSPOPE_BALANCE) && (ope->overdraft == TRUE) && (PREFS->custom_colors == TRUE) )
+		if( (column == LST_DSPOPE_BALANCE) && (PREFS->custom_colors == TRUE) && (ope->dspflags & TXN_DSPFLG_OVER) )
 		{
 			color = PREFS->color_warn;
 		}
@@ -505,6 +512,12 @@ gchar *color;
 			NULL);
 	//}
 
+	//test
+	if( (column == LST_DSPOPE_BALANCE) && (ope->dspflags & TXN_DSPFLG_LOWBAL))
+		g_object_set(renderer, "weight", PANGO_WEIGHT_BOLD, NULL);
+	else
+		g_object_set(renderer, "weight", PANGO_WEIGHT_NORMAL, NULL);
+
 	
 }
 
@@ -543,11 +556,12 @@ gchar *color, *text;
 		}
 		else
 		{
-			// #1673902 add a visual marker for uncategorized txn 
-			if( data->warnnocategory == TRUE )
+			//#1673902 add a visual marker for uncategorized txn
+			//#1844881 but not for internal xfer
+			if( (data->warnnocategory == TRUE) && !(ope->flags & OF_INTXFER) )
 			{
 				color = PREFS->color_warn;
-				text = _("- this need a category -");
+				text = _("- this needs a category -");
 			}
 		}
 	}
@@ -572,7 +586,7 @@ GString *node;
 const gchar *format;
 Transaction *ope;
 gchar datebuf[16];
-gchar *info, *payeename, *categoryname;
+gchar *payeename, *categoryname;
 gdouble amount, samount;
 Payee *payee;
 Category *category;
@@ -583,14 +597,18 @@ char amountbuf[G_ASCII_DTOSTR_BUF_SIZE];
 	
 	node = g_string_new(NULL);
 
-	//title line
 	if(clipboard)
+	{
 		g_string_append (node, "date\tpaymode\tinfo\tpayee\tmemo\tamount\tcategory\ttags\n");
+		format = "%s\t%d\t%s\t%s\t%s\t%s\t%s\t%s\n";
+	}
 	else
+	{
 		g_string_append (node, "date;paymode;info;payee;memo;amount;category;tags\n");
+		format = "%s;%d;%s;%s;%s;%s;%s;%s\n";
+	}
 	
 	model = gtk_tree_view_get_model(treeview);
-
 	valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);
 	while (valid)
 	{
@@ -601,8 +619,6 @@ char amountbuf[G_ASCII_DTOSTR_BUF_SIZE];
 
 		hb_sprint_date(datebuf, ope->date);
 
-		info = ope->info;
-		if(info == NULL) info = "";
 		payee = da_pay_get(ope->kpay);
 		payeename = (payee->name == NULL) ? "" : payee->name;
 		category = da_cat_get(ope->kcat);
@@ -618,17 +634,17 @@ char amountbuf[G_ASCII_DTOSTR_BUF_SIZE];
 		//g_ascii_dtostr (amountbuf, sizeof (amountbuf), ope->amount);
 		//#1750257 use locale numdigit
 		//g_ascii_formatd (amountbuf, sizeof (amountbuf), "%.2f", ope->amount);
+		//TODO: we should use the currency fmt here
 		g_snprintf(amountbuf, sizeof (amountbuf), "%.2f", amount);
 
 		DB( g_print("amount = %f '%s'\n", amount, amountbuf) );
 
-		format = (clipboard == TRUE) ? "%s\t%d\t%s\t%s\t%s\t%s\t%s\t%s\n" : "%s;%d;%s;%s;%s;%s;%s;%s\n";
 		g_string_append_printf(node, format,
 				datebuf,
 				ope->paymode,
-				info,
+				ope->info != NULL ? ope->info : "",
 				payeename,
-				ope->memo,
+				ope->memo != NULL ? ope->memo : "",
 				amountbuf,
 				categoryname != NULL ? categoryname : "",
 				tags != NULL ? tags : ""
@@ -1000,8 +1016,11 @@ gint i, col_id;
 		}
 	
 		gtk_menu_attach_to_widget (GTK_MENU (menu), button, NULL);
-		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
-				      ev->button, ev->time);
+		#if( (GTK_MAJOR_VERSION == 3) && (GTK_MINOR_VERSION >= 22) )
+		gtk_menu_popup_at_pointer(GTK_MENU (menu), NULL);
+		#else
+		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, ev->button, ev->time);
+		#endif
 	}
 
     return FALSE;
@@ -1130,6 +1149,54 @@ struct list_txn_data *data;
 }
 
 
+
+Transaction *list_txn_get_surround_transaction(GtkTreeView *treeview, Transaction **prev, Transaction **next)
+{
+GtkTreeModel *model;
+GtkTreeIter iter;
+GtkTreeIter *previter, *nextiter;
+GList *list;
+Transaction *ope;
+
+	ope = NULL;
+
+	model = gtk_tree_view_get_model(treeview);
+	list = gtk_tree_selection_get_selected_rows(gtk_tree_view_get_selection(treeview), &model);
+	if(list != NULL)
+	{
+		gtk_tree_model_get_iter(model, &iter, list->data);
+		gtk_tree_model_get(model, &iter, MODEL_TXN_POINTER, &ope, -1);
+
+		if(prev != NULL)
+		{
+			previter = gtk_tree_iter_copy(&iter);
+			if( gtk_tree_model_iter_previous(model, previter) )
+			{
+				gtk_tree_model_get(model, previter, MODEL_TXN_POINTER, prev, -1);
+			}
+			gtk_tree_iter_free(previter);
+		}
+
+		if(next != NULL)
+		{
+			nextiter = gtk_tree_iter_copy(&iter);
+			if( gtk_tree_model_iter_next(model, nextiter) )
+			{
+				gtk_tree_model_get(model, nextiter, MODEL_TXN_POINTER, next, -1);
+			}
+			gtk_tree_iter_free(nextiter);
+		}
+	}
+
+	g_list_foreach(list, (GFunc)gtk_tree_path_free, NULL);
+	g_list_free(list);
+
+	return ope;
+}
+
+
+
+
 Transaction *list_txn_get_active_transaction(GtkTreeView *treeview)
 {
 GtkTreeModel *model;
