@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2019 Maxime DOYEN
+ *  Copyright (C) 1995-2020 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -48,11 +48,6 @@ extern struct HomeBank *GLOBALS;
 extern struct Preferences *PREFS;
 
 
-static void repbudget_action_viewlist(GtkAction *action, gpointer user_data);
-static void repbudget_action_viewstack(GtkAction *action, gpointer user_data);
-static void repbudget_action_detail(GtkAction *action, gpointer user_data);
-static void repbudget_action_refresh(GtkAction *action, gpointer user_data);
-
 static void repbudget_date_change(GtkWidget *widget, gpointer user_data);
 static void repbudget_range_change(GtkWidget *widget, gpointer user_data);
 static void repbudget_toggle_detail(GtkWidget *widget, gpointer user_data);
@@ -72,51 +67,8 @@ extern gchar *CYA_CATSUBCAT[];
 extern gchar *CYA_KIND[];
 
 
-static GtkRadioActionEntry radio_entries[] = {
-  { "List"    , ICONNAME_HB_VIEW_LIST , N_("List")  , NULL,    N_("View results as list")      , 0 },
-  { "Stack"   , ICONNAME_HB_VIEW_STACK, N_("Stack") , NULL,    N_("View results as stack bars"), 1 },
-};
-static guint n_radio_entries = G_N_ELEMENTS (radio_entries);
-
-
-static GtkActionEntry entries[] = {
-  { "Refresh" , ICONNAME_HB_REFRESH, N_("Refresh"), NULL,   N_("Refresh results"), G_CALLBACK (repbudget_action_refresh) },
-
-//  { "Export"  , ICONNAME_HB_FILE_EXPORT , N_("Export")  , NULL,   N_("Export as CSV"), G_CALLBACK (repbudget_action_export) },
-};
-static guint n_entries = G_N_ELEMENTS (entries);
-
-
-static GtkToggleActionEntry toggle_entries[] = {
-  { "Detail", ICONNAME_HB_OPE_SHOW,                    /* name, icon-name */
-     N_("Detail"), NULL,                    /* label, accelerator */
-    N_("Toggle detail"),                                    /* tooltip */
-    G_CALLBACK (repbudget_action_detail),
-    FALSE },                                    /* is_active */
-
-};
-static guint n_toggle_entries = G_N_ELEMENTS (toggle_entries);
-
-
-static const gchar *ui_info =
-"<ui>"
-"  <toolbar name='ToolBar'>"
-"    <toolitem action='List'/>"
-"    <toolitem action='Stack'/>"
-"      <separator/>"
-"    <toolitem action='Detail'/>"
-"      <separator/>"
-"    <toolitem action='Refresh'/>"
-"      <separator/>"
-//"    <toolitem action='Export'/>"
-//		replaced by a menubutton
-"  </toolbar>"
-"</ui>";
-
-
-
 /* action functions -------------------- */
-static void repbudget_action_viewlist(GtkAction *action, gpointer user_data)
+static void repbudget_action_viewlist(GtkToolButton *toolbutton, gpointer user_data)
 {
 struct repbudget_data *data = user_data;
 
@@ -124,7 +76,7 @@ struct repbudget_data *data = user_data;
 	repbudget_sensitive(data->window, NULL);
 }
 
-static void repbudget_action_viewstack(GtkAction *action, gpointer user_data)
+static void repbudget_action_viewstack(GtkToolButton *toolbutton, gpointer user_data)
 {
 struct repbudget_data *data = user_data;
 
@@ -133,45 +85,6 @@ struct repbudget_data *data = user_data;
 }
 
 
-static void repbudget_action_mode (GtkRadioAction *action, GtkRadioAction *current, gpointer user_data)
-{
-gint value;
-
-	value = gtk_radio_action_get_current_value(GTK_RADIO_ACTION(action));
-	switch( value )
-	{
-		case 0:
-			repbudget_action_viewlist(GTK_ACTION(action), user_data);
-			break;
-		case 1:
-			repbudget_action_viewstack(GTK_ACTION(action), user_data);
-			break;
-	}
-}
-
-
-static void repbudget_action_detail(GtkAction *action, gpointer user_data)
-{
-struct repbudget_data *data = user_data;
-
-	repbudget_toggle_detail(data->window, NULL);
-}
-
-
-static void repbudget_action_refresh(GtkAction *action, gpointer user_data)
-{
-struct repbudget_data *data = user_data;
-
-	repbudget_compute(data->window, NULL);
-}
-
-/*static void repbudget_action_export(GtkAction *action, gpointer user_data)
-{
-struct repbudget_data *data = user_data;
-
-	repbudget_export_csv(data->window, NULL);
-}*/
-
 /* ======================== */
 
 
@@ -968,7 +881,6 @@ struct repbudget_data *data;
 static void repbudget_sensitive(GtkWidget *widget, gpointer user_data)
 {
 struct repbudget_data *data;
-GtkAction *action;
 gboolean visible, sensitive;
 gint page;
 
@@ -978,16 +890,13 @@ gint page;
 
 	page = gtk_notebook_get_current_page(GTK_NOTEBOOK(data->GR_result));
 
-	visible = page == 0 ? TRUE : FALSE;
-	action = gtk_ui_manager_get_action(data->ui, "/ToolBar/Detail");
-	gtk_action_set_visible (action, visible);
-	//sensitive = gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(data->LV_report)), NULL, NULL);
-	//gtk_action_set_sensitive(action, sensitive);
-	//action = gtk_ui_manager_get_action(data->ui, "/ToolBar/Export");
-	//gtk_action_set_visible (action, visible);
+	visible = (page == 0) ? TRUE : FALSE;
+	hb_widget_visible (data->BT_detail, visible);
 	hb_widget_visible (data->BT_export, visible);
-	
-	sensitive = gtk_tree_model_iter_n_children(gtk_tree_view_get_model(GTK_TREE_VIEW(data->LV_detail)), NULL) > 0 ? TRUE : FALSE;
+
+	page = gtk_tree_model_iter_n_children(gtk_tree_view_get_model(GTK_TREE_VIEW(data->LV_detail)), NULL);
+
+	sensitive = ((page > 0) && data->detail) ? TRUE : FALSE;
 	gtk_widget_set_sensitive(data->MI_detailtoclip, sensitive);
 	gtk_widget_set_sensitive(data->MI_detailtocsv, sensitive);
 
@@ -1026,8 +935,8 @@ static void repbudget_setup(struct repbudget_data *data)
 	data->legend = 1;
 
 	/* 3.4 : make int transfer out of stats */
-	data->filter->option[FILTER_PAYMODE] = 1;
-	data->filter->paymode[PAYMODE_INTXFER] = FALSE;
+	data->filter->option[FILTER_TYPE] = 2;
+	data->filter->type = FLT_TYPE_INTXFER;
 
 	filter_preset_daterange_set(data->filter, PREFS->date_range_rep, 0);
 	
@@ -1063,6 +972,86 @@ guint key = -1;
 }
 
 
+static GtkWidget *
+repbudget_toolbar_create(struct repbudget_data *data)
+{
+GtkWidget *toolbar, *button;
+
+	toolbar = gtk_toolbar_new();
+
+	button = (GtkWidget *)gtk_radio_tool_button_new(NULL);
+	data->BT_list = button;
+	g_object_set (button, "icon-name", ICONNAME_HB_VIEW_LIST, NULL);
+	gtk_widget_set_tooltip_text(button, _("View results as list"));
+	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(button), -1);
+
+	button = (GtkWidget *)gtk_radio_tool_button_new_from_widget(GTK_RADIO_TOOL_BUTTON(button));
+	data->BT_stack = button;
+	g_object_set (button, "icon-name", ICONNAME_HB_VIEW_STACK, NULL);
+	gtk_widget_set_tooltip_text(button, _("View results as stack bars"));
+	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(button), -1);
+	
+	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(), -1);
+
+	button = gtk_widget_new(GTK_TYPE_TOGGLE_TOOL_BUTTON,
+		"icon-name", ICONNAME_HB_OPE_SHOW,
+		"tooltip-text", _("Toggle detail"),
+	    "active", PREFS->budg_showdetail,
+		NULL);
+	data->BT_detail = button;
+	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(button), -1);
+
+	button = hbtk_toolbar_add_toolbutton(GTK_TOOLBAR(toolbar), ICONNAME_HB_REFRESH, _("Refresh results"));
+	data->BT_refresh = button;
+	
+	GtkWidget *menu, *menuitem;
+
+	menu = gtk_menu_new ();
+	//gtk_widget_set_halign (menu, GTK_ALIGN_END);
+
+	menuitem = gtk_menu_item_new_with_mnemonic (_("_Result to clipboard"));
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	g_signal_connect (G_OBJECT (menuitem), "activate", G_CALLBACK (repbudget_export_result_clipboard), data);
+
+	menuitem = gtk_menu_item_new_with_mnemonic (_("_Result to CSV"));
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	g_signal_connect (G_OBJECT (menuitem), "activate", G_CALLBACK (repbudget_export_result_csv), data);
+
+	menuitem = gtk_menu_item_new_with_mnemonic (_("_Detail to clipboard"));
+	data->MI_detailtoclip = menuitem;
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	g_signal_connect (G_OBJECT (menuitem), "activate", G_CALLBACK (repbudget_export_detail_clipboard), data);
+
+	menuitem = gtk_menu_item_new_with_mnemonic (_("_Detail to CSV"));
+	data->MI_detailtocsv = menuitem;
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	g_signal_connect (G_OBJECT (menuitem), "activate", G_CALLBACK (repbudget_export_detail_csv), data);
+
+	gtk_widget_show_all (menu);
+
+
+	
+	button = gtk_menu_button_new();
+	data->BT_export = button;
+	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET(button)), GTK_STYLE_CLASS_FLAT);
+
+	//gtk_menu_button_set_direction (GTK_MENU_BUTTON(widget), GTK_ARROW_DOWN);
+	//gtk_widget_set_halign (widget, GTK_ALIGN_END);
+	GtkWidget *image = gtk_image_new_from_icon_name (ICONNAME_HB_FILE_EXPORT, GTK_ICON_SIZE_LARGE_TOOLBAR);
+	g_object_set (button, "image", image, "popup", GTK_MENU(menu),  NULL);
+
+	GtkToolItem *toolitem = gtk_tool_item_new();
+	gtk_container_add (GTK_CONTAINER(toolitem), button);
+	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(toolitem), -1);
+
+	
+	
+	
+	return toolbar;
+}
+
+
+
 /*
 **
 */
@@ -1104,10 +1093,6 @@ struct WinGeometry *wg;
 GtkWidget *window, *mainvbox, *hbox, *vbox, *notebook, *treeview;
 GtkWidget *label, *widget, *table, *entry;
 gint row;
-GtkUIManager *ui;
-GtkActionGroup *actions;
-GtkAction *action;
-GError *error = NULL;
 
 	data = g_malloc0(sizeof(struct repbudget_data));
 	if(!data) return NULL;
@@ -1210,93 +1195,11 @@ GError *error = NULL;
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
 
-	//ui manager
-	actions = gtk_action_group_new ("default");
-
-	//as we use gettext
-   	gtk_action_group_set_translation_domain(actions, GETTEXT_PACKAGE);
-
-	// data to action callbacks is set here (data)
-	gtk_action_group_add_radio_actions (actions, radio_entries, n_radio_entries, 0, G_CALLBACK(repbudget_action_mode), data);
-
-	gtk_action_group_add_actions (actions, entries, n_entries, data);
-
-	gtk_action_group_add_toggle_actions (actions,
-					   toggle_entries, n_toggle_entries,
-					   data);
-
-
-	/* set which action should have priority in the toolbar */
-	//action = gtk_action_group_get_action(actions, "List");
-	//g_object_set(action, "is_important", TRUE, NULL);
-
-	//action = gtk_action_group_get_action(actions, "Stack");
-	//g_object_set(action, "is_important", TRUE, NULL);
-
-	action = gtk_action_group_get_action(actions, "Detail");
-	//g_object_set(action, "is_important", TRUE, NULL);
-	g_object_set(action, "active", PREFS->budg_showdetail, NULL);
-
-	//action = gtk_action_group_get_action(actions, "Refresh");
-	//g_object_set(action, "is_important", TRUE, NULL);
-
-
- 	ui = gtk_ui_manager_new ();
-	gtk_ui_manager_insert_action_group (ui, actions, 0);
-	gtk_window_add_accel_group (GTK_WINDOW (window), gtk_ui_manager_get_accel_group (ui));
-
-	if (!gtk_ui_manager_add_ui_from_string (ui, ui_info, -1, &error))
-	{
-		g_message ("building UI failed: %s", error->message);
-		g_error_free (error);
-	}
-
-	data->ui = ui;
-
 	//toolbar
-	data->TB_bar = gtk_ui_manager_get_widget (ui, "/ToolBar");
-	gtk_box_pack_start (GTK_BOX (vbox), data->TB_bar, FALSE, FALSE, 0);
-
-	//add export menu button
-	GtkToolItem *toolitem;
-	GtkWidget *menu, *menuitem, *image;
-
-	menu = gtk_menu_new ();
-	//gtk_widget_set_halign (menu, GTK_ALIGN_END);
-
-	menuitem = gtk_menu_item_new_with_mnemonic (_("_Result to clipboard"));
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	g_signal_connect (G_OBJECT (menuitem), "activate", G_CALLBACK (repbudget_export_result_clipboard), data);
-
-	menuitem = gtk_menu_item_new_with_mnemonic (_("_Result to CSV"));
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	g_signal_connect (G_OBJECT (menuitem), "activate", G_CALLBACK (repbudget_export_result_csv), data);
-
-	menuitem = gtk_menu_item_new_with_mnemonic (_("_Detail to clipboard"));
-	data->MI_detailtoclip = menuitem;
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	g_signal_connect (G_OBJECT (menuitem), "activate", G_CALLBACK (repbudget_export_detail_clipboard), data);
-
-	menuitem = gtk_menu_item_new_with_mnemonic (_("_Detail to CSV"));
-	data->MI_detailtocsv = menuitem;
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	g_signal_connect (G_OBJECT (menuitem), "activate", G_CALLBACK (repbudget_export_detail_csv), data);
-
-	gtk_widget_show_all (menu);
-
-	widget = gtk_menu_button_new();
-	data->BT_export = widget;
-	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET(widget)), GTK_STYLE_CLASS_FLAT);
-
-	//gtk_menu_button_set_direction (GTK_MENU_BUTTON(widget), GTK_ARROW_DOWN);
-	//gtk_widget_set_halign (widget, GTK_ALIGN_END);
-	image = gtk_image_new_from_icon_name (ICONNAME_HB_FILE_EXPORT, GTK_ICON_SIZE_LARGE_TOOLBAR);
-	g_object_set (widget, "image", image, "popup", GTK_MENU(menu),  NULL);
-
-	toolitem = gtk_tool_item_new();
-	gtk_container_add (GTK_CONTAINER(toolitem), widget);
-	gtk_toolbar_insert(GTK_TOOLBAR(data->TB_bar), GTK_TOOL_ITEM(toolitem), -1);
-
+	widget = repbudget_toolbar_create(data);
+	data->TB_bar = widget;
+	gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 0);
+	
 	//infos
 	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, SPACING_SMALL);
 	gtk_container_set_border_width (GTK_CONTAINER(hbox), SPACING_SMALL);
@@ -1305,7 +1208,7 @@ GError *error = NULL;
 	widget = make_label(NULL, 0.5, 0.5);
 	gimp_label_set_attributes (GTK_LABEL (widget), PANGO_ATTR_SCALE,  PANGO_SCALE_SMALL, -1);
 	data->TX_daterange = widget;
-	gtk_box_pack_start (GTK_BOX (hbox), widget, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 0);
 
 
 	entry = gtk_label_new(NULL);
@@ -1388,6 +1291,14 @@ GError *error = NULL;
     data->handler_id[HID_REPBUDGET_MINDATE] = g_signal_connect (data->PO_mindate, "changed", G_CALLBACK (repbudget_date_change), (gpointer)data);
     data->handler_id[HID_REPBUDGET_MAXDATE] = g_signal_connect (data->PO_maxdate, "changed", G_CALLBACK (repbudget_date_change), (gpointer)data);
 
+	g_signal_connect (G_OBJECT (data->BT_list), "clicked", G_CALLBACK (repbudget_action_viewlist), (gpointer)data);
+	g_signal_connect (G_OBJECT (data->BT_stack), "clicked", G_CALLBACK (repbudget_action_viewstack), (gpointer)data);
+	g_signal_connect (G_OBJECT (data->BT_detail), "clicked", G_CALLBACK (repbudget_toggle_detail), (gpointer)data);
+	g_signal_connect (G_OBJECT (data->BT_refresh), "clicked", G_CALLBACK (repbudget_compute), (gpointer)data);
+	//export is a menu
+
+
+	
 	g_signal_connect (gtk_tree_view_get_selection(GTK_TREE_VIEW(data->LV_report)), "changed", G_CALLBACK (repbudget_selection), NULL);
 
 	g_signal_connect (GTK_TREE_VIEW(data->LV_detail), "row-activated", G_CALLBACK (repbudget_detail_onRowActivated), NULL);
@@ -1537,7 +1448,6 @@ gchar *entry1, *entry2;
 
 	retval = hb_string_utf8_compare(entry1, entry2);
 
-
 	g_free(entry2);
 	g_free(entry1);
 
