@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2019 Maxime DOYEN
+ *  Copyright (C) 1995-2020 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -47,9 +47,6 @@ extern struct Preferences *PREFS;
 
 
 /* prototypes */
-static void repvehicle_action_refresh(GtkAction *action, gpointer user_data);
-static void repvehicle_action_export(GtkAction *action, gpointer user_data);
-
 static void repvehicle_export_csv(GtkWidget *widget, gpointer user_data);
 static void repvehicle_date_change(GtkWidget *widget, gpointer user_data);
 static void repvehicle_range_change(GtkWidget *widget, gpointer user_data);
@@ -61,22 +58,6 @@ static gboolean repvehicle_window_dispose(GtkWidget *widget, GdkEvent *event, gp
 static GtkWidget *create_list_repvehicle(void);
 
 
-static GtkActionEntry entries[] = {
-  { "Refresh" , ICONNAME_HB_REFRESH   , N_("Refresh"), NULL,   N_("Refresh results"), G_CALLBACK (repvehicle_action_refresh) },
-
-  { "Export" , ICONNAME_HB_FILE_EXPORT, N_("Export")  , NULL,   N_("Export as CSV"), G_CALLBACK (repvehicle_action_export) },
-};
-static guint n_entries = G_N_ELEMENTS (entries);
-
-static const gchar *ui_info =
-"<ui>"
-"  <toolbar name='ToolBar'>"
-"    <toolitem action='Refresh'/>"
-"      <separator/>"
-"    <toolitem action='Export'/>"
-"  </toolbar>"
-"</ui>";
-
 static void repvehicle_date_change(GtkWidget *widget, gpointer user_data)
 {
 struct repvehicle_data *data;
@@ -102,14 +83,14 @@ struct repvehicle_data *data;
 }
 
 
-static void repvehicle_action_refresh(GtkAction *action, gpointer user_data)
+static void repvehicle_action_refresh(GtkToolButton *toolbutton, gpointer user_data)
 {
 struct repvehicle_data *data = user_data;
 
 	repvehicle_compute(data->window, NULL);
 }
 
-static void repvehicle_action_export(GtkAction *action, gpointer user_data)
+static void repvehicle_action_export(GtkToolButton *toolbutton, gpointer user_data)
 {
 struct repvehicle_data *data = user_data;
 
@@ -320,7 +301,7 @@ guint32 catkey;
 				item->amount = hb_amount_base(ope->amount, ope->kcur);
 
 				item = repvehicle_eval_memofield(item, ope->memo);
-				data->vehicle_list = g_list_append(data->vehicle_list, item);
+				data->vehicle_list = g_list_prepend(data->vehicle_list, item);
 				DB( g_print(" -> store acc=%d '%s' %.2f\n", ope->kacc, ope->memo, ope->amount) );
 			}
 		}
@@ -349,7 +330,7 @@ guint32 catkey;
 					item->amount = hb_amount_base(split->amount, ope->kcur);
 
 					item = repvehicle_eval_memofield(item, split->memo);
-					data->vehicle_list = g_list_append(data->vehicle_list, item);
+					data->vehicle_list = g_list_prepend(data->vehicle_list, item);
 					DB( g_print(" -> store split part acc=%d '%s' %.2f\n", ope->kacc, split->memo, split->amount) );
 				}
 			}
@@ -614,8 +595,8 @@ static void repvehicle_setup(struct repvehicle_data *data)
 	filter_reset(data->filter);
 
 	/* 3.4 : make int transfer out of stats */
-	data->filter->option[FILTER_PAYMODE] = 1;
-	data->filter->paymode[PAYMODE_INTXFER] = FALSE;
+	data->filter->option[FILTER_TYPE] = 2;
+	data->filter->type = FLT_TYPE_INTXFER;
 
 	filter_preset_daterange_set(data->filter, PREFS->date_range_rep, 0);
 	
@@ -640,10 +621,27 @@ static void repvehicle_setup(struct repvehicle_data *data)
 }
 
 
-/*
-**
-*/
-static gboolean repvehicle_window_dispose(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+
+
+static GtkWidget *
+repvehicle_toolbar_create(struct repvehicle_data *data)
+{
+GtkWidget *toolbar, *button;
+
+	toolbar = gtk_toolbar_new();
+
+	button = hbtk_toolbar_add_toolbutton(GTK_TOOLBAR(toolbar), ICONNAME_HB_REFRESH, _("Refresh results"));
+	data->BT_refresh = button;
+
+	button = hbtk_toolbar_add_toolbutton(GTK_TOOLBAR(toolbar), ICONNAME_HB_FILE_EXPORT, _("Export as CSV"));
+	data->BT_export = button;
+	
+	return toolbar;
+}
+
+
+static gboolean
+repvehicle_window_dispose(GtkWidget *widget, GdkEvent *event, gpointer user_data)
 {
 struct repvehicle_data *data = user_data;
 struct WinGeometry *wg;
@@ -681,10 +679,6 @@ struct WinGeometry *wg;
 GtkWidget *window, *mainvbox, *hbox, *vbox, *treeview;
 GtkWidget *label, *widget, *table;
 gint row, col;
-GtkUIManager *ui;
-GtkActionGroup *actions;
-//GtkAction *action;
-GError *error = NULL;
 
 	data = g_malloc0(sizeof(struct repvehicle_data));
 	if(!data) return NULL;
@@ -774,37 +768,10 @@ GError *error = NULL;
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
 
-	//ui manager
-	actions = gtk_action_group_new ("default");
-
-	//as we use gettext
-   	gtk_action_group_set_translation_domain(actions, GETTEXT_PACKAGE);
-
-	// data to action callbacks is set here (data)
-	gtk_action_group_add_actions (actions, entries, n_entries, data);
-
-	/* set which action should have priority in the toolbar */
-	//action = gtk_action_group_get_action(actions, "Refresh");
-	//g_object_set(action, "is_important", TRUE, NULL);
-
-
-	ui = gtk_ui_manager_new ();
-	gtk_ui_manager_insert_action_group (ui, actions, 0);
-	gtk_window_add_accel_group (GTK_WINDOW (window), gtk_ui_manager_get_accel_group (ui));
-
-	if (!gtk_ui_manager_add_ui_from_string (ui, ui_info, -1, &error))
-	{
-		g_message ("building UI failed: %s", error->message);
-		g_error_free (error);
-	}
-
-	data->ui = ui;
-	data->actions = actions;
-
-	//toolbar
-	data->TB_bar = gtk_ui_manager_get_widget (ui, "/ToolBar");
-	gtk_box_pack_start (GTK_BOX (vbox), data->TB_bar, FALSE, FALSE, 0);
-
+	widget = repvehicle_toolbar_create(data);
+	data->TB_bar = widget;
+	gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 0);
+	
 
 	// total
 	table = gtk_grid_new ();
@@ -894,7 +861,10 @@ GError *error = NULL;
 
 	data->handler_id[HID_REPVEHICLE_VEHICLE] = g_signal_connect (data->PO_cat, "changed", G_CALLBACK (repvehicle_compute), NULL);
 
+	g_signal_connect (G_OBJECT (data->BT_refresh), "clicked", G_CALLBACK (repvehicle_action_refresh), (gpointer)data);
+	g_signal_connect (G_OBJECT (data->BT_export) , "clicked", G_CALLBACK (repvehicle_action_export), (gpointer)data);
 
+	
 	//setup, init and show window
 	repvehicle_setup(data);
 
