@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2019 Maxime DOYEN
+ *  Copyright (C) 1995-2020 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -44,15 +44,6 @@ extern struct Preferences *PREFS;
 extern gchar *CYA_ABMONTHS[];
 
 
-static gchar *ui_bud_manage_getcsvbudgetstr(Category *item);
-static void ui_bud_manage_update(GtkWidget *treeview, gpointer user_data);
-static void ui_bud_manage_set(GtkWidget *widget, gpointer user_data);
-static void ui_bud_manage_getlast(struct ui_bud_manage_data *data);
-static void ui_bud_manage_selection_change(GtkWidget *treeview, gpointer user_data);
-static void ui_bud_manage_toggle(GtkRadioButton *radiobutton, gpointer user_data);
-static void ui_bud_manage_selection(GtkTreeSelection *treeselection, gpointer user_data);
-
-
 /* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
 
 /*
@@ -91,7 +82,12 @@ gchar *iconname = NULL;
 	// get the transaction
 	gtk_tree_model_get(model, iter, LST_DEFCAT_DATAS, &item, -1);
 
-	iconname = ( item->flags & GF_BUDGET ) ? ICONNAME_HB_OPE_BUDGET : NULL;
+	//5.3 added
+	if( item->flags & GF_FORCED )
+		iconname = ICONNAME_HB_OPE_FORCED;
+	else
+		if( item->flags & GF_BUDGET )
+			iconname = ICONNAME_HB_OPE_BUDGET;
 
 	g_object_set(renderer, "icon-name", iconname, NULL);
 }
@@ -120,9 +116,6 @@ gchar type;
 
 	type = category_get_type_char(entry);
 
-	#if MYDEBUG
-		string = g_markup_printf_escaped("%s ::(f=%d, %c)", name, entry->flags, type );
-	#else
 	if(entry->key == 0)
 		string = g_strdup(name);
 	else
@@ -142,7 +135,6 @@ gchar type;
 				string = g_markup_printf_escaped(" %c <i>%s</i>", type, name);
 		}
 	}
-	#endif
 
 	g_object_set(renderer, "markup", string, NULL);
 
@@ -150,6 +142,26 @@ gchar type;
 }
 
 
+#if MYDEBUG == 1
+static void
+ui_bud_listview_cell_data_function_debug (GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data)
+{
+Category *entry;
+gchar *string;
+gchar type;
+
+	gtk_tree_model_get(model, iter, LST_DEFCAT_DATAS, &entry, -1);
+
+	type = category_get_type_char(entry);
+
+	string = g_markup_printf_escaped("f:%d t:%c", entry->flags, type );
+	g_object_set(renderer, "markup", string, NULL);
+	g_free(string);
+}
+#endif	
+
+
+
 static gboolean ui_bud_listview_search_equal_func (GtkTreeModel *model,
                                gint column,
                                const gchar *key,
@@ -220,14 +232,25 @@ GtkTreeViewColumn  *column;
 
 	gtk_tree_view_set_grid_lines (GTK_TREE_VIEW (treeview), PREFS->grid_lines);
 	gtk_tree_view_set_enable_tree_lines(GTK_TREE_VIEW (treeview), TRUE);
-	
-	/* column 1 */
-	renderer = gtk_cell_renderer_text_new ();
-	g_object_set(renderer, 
-		"ellipsize", PANGO_ELLIPSIZE_END,
-	    "ellipsize-set", TRUE,
-	    NULL);	
 
+	#if MYDEBUG == 1
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new();
+	gtk_tree_view_column_pack_start(column, renderer, TRUE);
+	gtk_tree_view_column_set_cell_data_func(column, renderer, ui_bud_listview_cell_data_function_debug, NULL, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), column);
+	#endif
+	
+	/* icon column */
+	column = gtk_tree_view_column_new();
+	renderer = gtk_cell_renderer_pixbuf_new ();
+	//gtk_cell_renderer_set_fixed_size(renderer, GLOBALS->lst_pixbuf_maxwidth, -1);
+	gtk_tree_view_column_pack_start(column, renderer, TRUE);
+	gtk_tree_view_column_set_cell_data_func(column, renderer, ui_bud_listview_icon_cell_data_function, NULL, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), column);
+	
+	/* category name */
+	renderer = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new();
 	gtk_tree_view_column_set_title(column, _("Category"));
 	gtk_tree_view_column_pack_start(column, renderer, TRUE);
@@ -238,14 +261,7 @@ GtkTreeViewColumn  *column;
 	gtk_tree_view_column_set_resizable(column, TRUE);
 	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), column);
 
-	/* icon column */
-	column = gtk_tree_view_column_new();
-	renderer = gtk_cell_renderer_pixbuf_new ();
-	//gtk_cell_renderer_set_fixed_size(renderer, GLOBALS->lst_pixbuf_maxwidth, -1);
-	gtk_tree_view_column_pack_start(column, renderer, TRUE);
-	gtk_tree_view_column_set_cell_data_func(column, renderer, ui_bud_listview_icon_cell_data_function, NULL, NULL);
-	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), column);
-
+	gtk_tree_view_set_expander_column(GTK_TREE_VIEW(treeview), column);
 	
 	gtk_tree_view_set_search_equal_func(GTK_TREE_VIEW(treeview), ui_bud_listview_search_equal_func, NULL, NULL);
 
@@ -380,9 +396,9 @@ const gchar *encoding;
 	//data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(widget), GTK_TYPE_WINDOW)), "inst_data");
 
 
-	DB( g_print("(ui_bud_manage) load csv - data %p\n", data) );
+	DB( g_print("\n[ui-budget] load csv - data %p\n", data) );
 
-	if( ui_file_chooser_csv(GTK_WINDOW(data->window), GTK_FILE_CHOOSER_ACTION_OPEN, &filename, NULL) == TRUE )
+	if( ui_file_chooser_csv(GTK_WINDOW(data->dialog), GTK_FILE_CHOOSER_ACTION_OPEN, &filename, NULL) == TRUE )
 	{
 		DB( g_print(" + filename is %s\n", filename) );
 
@@ -501,7 +517,7 @@ const gchar *encoding;
 
 			if( error == TRUE )
 			{
-				ui_dialog_msg_infoerror(GTK_WINDOW(data->window), GTK_MESSAGE_ERROR,
+				ui_dialog_msg_infoerror(GTK_WINDOW(data->dialog), GTK_MESSAGE_ERROR,
 					_("File format error"),
 					_("The CSV file must contains the exact numbers of column,\nseparated by a semi-colon, please see the help for more details.")
 					);
@@ -524,11 +540,11 @@ GtkTreeIter	iter, child;
 gboolean valid;
 GIOChannel *io;
 
-	DB( g_print("(ui_bud_manage) save csv\n") );
+	DB( g_print("\n[ui-budget] save csv\n") );
 
 	//data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(widget, GTK_TYPE_WINDOW)), "inst_data");
 
-	if( ui_file_chooser_csv(GTK_WINDOW(data->window), GTK_FILE_CHOOSER_ACTION_SAVE, &filename, NULL) == TRUE )
+	if( ui_file_chooser_csv(GTK_WINDOW(data->dialog), GTK_FILE_CHOOSER_ACTION_SAVE, &filename, NULL) == TRUE )
 	{
 
 		DB( g_print(" + filename is %s\n", filename) );
@@ -608,7 +624,7 @@ static void ui_bud_manage_expand_all(GtkWidget *widget, gpointer user_data)
 struct ui_bud_manage_data *data;
 
 	data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(widget, GTK_TYPE_WINDOW)), "inst_data");
-	DB( g_print("\n(ui_bud_manage) expand all (data=%x)\n", (guint)data) );
+	DB( g_print("\n[ui-budget] expand all (data=%p)\n", data) );
 
 	gtk_tree_view_expand_all(GTK_TREE_VIEW(data->LV_cat));
 
@@ -620,12 +636,13 @@ static void ui_bud_manage_collapse_all(GtkWidget *widget, gpointer user_data)
 struct ui_bud_manage_data *data;
 
 	data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(widget, GTK_TYPE_WINDOW)), "inst_data");
-	DB( g_print("\n(ui_bud_manage) collapse all (data=%x)\n", (guint)data) );
+	DB( g_print("\n[ui-budget] collapse all (data=%p)\n", data) );
 
 	gtk_tree_view_collapse_all(GTK_TREE_VIEW(data->LV_cat));
 
 }
 
+
 static void ui_bud_manage_update(GtkWidget *treeview, gpointer user_data)
 {
 struct ui_bud_manage_data *data;
@@ -634,7 +651,7 @@ gint i;
 
 	data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(treeview), GTK_TYPE_WINDOW)), "inst_data");
 
-	DB( g_print("\n(ui_bud_manage) update %x\n", (gint)data) );
+	DB( g_print("\n[ui-budget] update %p\n", data) );
 
 
 	name = FALSE;
@@ -660,7 +677,7 @@ gint i;
 #endif
 
 	custom = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->CM_type[1]));
-	DB( g_print(" custom = %d\n\n", custom) );
+	DB( g_print(" custom = %d\n", custom) );
 
 	sensitive = name == FALSE ? FALSE : custom == TRUE ? FALSE: TRUE;
 	gtk_widget_set_sensitive(data->spinner[0], sensitive);
@@ -672,6 +689,220 @@ gint i;
 		gtk_widget_set_sensitive(data->spinner[i+1], sensitive);
 	}
 
+
+
+	
+	DB( g_print(" -- end update %p\n", data) );
+
+	
+}
+
+
+static void
+ui_bud_manage_compute_total(GtkWidget *widget, gpointer user_data)
+{
+struct ui_bud_manage_data *data;
+GList *lcat, *list;
+gdouble value;
+gint j;
+
+	data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(widget), GTK_TYPE_WINDOW)), "inst_data");
+
+	DB( g_print("\n[ui-budget] compute total\n") );
+
+	data->totexp = 0;
+	data->totinc = 0;
+
+	lcat = list = category_glist_sorted(1);
+	while (list != NULL)
+	{
+	Category *item = list->data;
+
+		value = 0;
+		if(!(item->flags & GF_CUSTOM))
+		{
+			value += (12*item->budget[0]);
+		}
+		//otherwise	sum each month from mindate month
+		else
+		{
+			for(j=1;j<12;j++) {
+				value += item->budget[j];
+			}
+		}
+
+		if( value <= 0.0 )
+			data->totexp += value;
+		else
+			data->totinc += value;
+
+		list = g_list_next(list);
+	}
+	g_list_free(lcat);
+
+	hb_label_set_colvalue(GTK_LABEL(data->TX_totexp), data->totexp, GLOBALS->kcur, GLOBALS->minor);
+	hb_label_set_colvalue(GTK_LABEL(data->TX_totinc), data->totinc, GLOBALS->kcur, GLOBALS->minor);
+	hb_label_set_colvalue(GTK_LABEL(data->TX_totbal), data->totexp + data->totinc, GLOBALS->kcur, GLOBALS->minor);
+}
+
+
+
+static void
+ui_bud_manage_cb_forcemonitor_toggled(GtkWidget *widget, gpointer user_data)
+{
+struct ui_bud_manage_data *data;
+Category *item;
+	
+	DB( g_print("\n[ui-budget] forced toggled\n") );
+	
+	data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(widget), GTK_TYPE_WINDOW)), "inst_data");
+
+	item = data->cat;
+	if( item )
+	{
+		item->flags &= ~(GF_FORCED);
+		if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->CM_force)) && !(item->flags & GF_BUDGET) )
+			item->flags |= GF_FORCED;
+	
+		hbtk_listview_redraw_selected_row (GTK_TREE_VIEW(data->LV_cat));
+	}
+}
+
+
+static void
+ui_bud_manage_toggle(GtkRadioButton *radiobutton, gpointer user_data)
+{
+//struct ui_bud_manage_data *data;
+
+	//data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(radiobutton), GTK_TYPE_WINDOW)), "inst_data");
+
+	DB( g_print("\n[ui-budget] toggle\n") );
+
+	//ui_bud_manage_get(GTK_WIDGET(radiobutton), GINT_TO_POINTER(FIELD_TYPE));
+
+	//data->custom ^= 1;
+	ui_bud_manage_update(GTK_WIDGET(radiobutton), NULL);
+}
+
+
+static void ui_bud_manage_get(struct ui_bud_manage_data *data, Category *item)
+{
+
+	DB( g_print("\n[ui-budget] get") );
+
+	if( item != NULL )
+	{
+		item->flags &= ~(GF_CUSTOM|GF_BUDGET|GF_FORCED);
+
+		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->CM_type[0])) == FALSE)
+		{
+			item->flags |= GF_CUSTOM;
+		}
+
+		for(guint i=0;i<=12;i++)
+		{
+			gtk_spin_button_update(GTK_SPIN_BUTTON(data->spinner[i]));
+			item->budget[i] = gtk_spin_button_get_value(GTK_SPIN_BUTTON(data->spinner[i]));
+			if( item->budget[i] )
+				item->flags |= GF_BUDGET;
+		}
+
+		if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->CM_force)) && !(item->flags & GF_BUDGET) )
+		{
+			item->flags |= GF_FORCED;
+		}
+	}
+
+	DB( g_print(" -- end get") );
+
+}
+
+
+static gboolean ui_bud_manage_cb_budget_changed(GtkSpinButton *spinbutton, gpointer user_data)
+{
+struct ui_bud_manage_data *data;
+gboolean retval = FALSE;
+
+	data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(spinbutton), GTK_TYPE_WINDOW)), "inst_data");
+
+	DB( g_print("\n[ui-budget] budget changed\n") );
+
+	ui_bud_manage_get(data, data->cat);
+
+	hbtk_listview_redraw_selected_row (GTK_TREE_VIEW(data->LV_cat));
+	ui_bud_manage_compute_total(data->dialog, NULL);
+
+	DB( g_print(" -- end budget changed\n") );
+
+	return retval;
+}
+
+
+static void ui_bud_manage_getlast(struct ui_bud_manage_data *data)
+{
+Category *item;
+
+	DB( g_print("\n[ui-budget] getlast") );
+
+	item = data->lastcatitem;
+	if( item != NULL )
+	{
+	gushort old_flags = item->flags;
+	gdouble oldsum = 0.0;
+	gdouble sum = 0.0;
+		
+		for(guint i=0;i<=12;i++)
+		{
+			oldsum += item->budget[i];
+		}
+
+		ui_bud_manage_get(data, item);
+		
+		for(guint i=0;i<=12;i++)
+		{
+			sum += item->budget[i];
+		}
+
+		if( (old_flags != item->flags) || (oldsum != sum) )
+			data->change++;
+
+		hbtk_listview_redraw_selected_row (GTK_TREE_VIEW(data->LV_cat));
+		ui_bud_manage_compute_total(data->dialog, NULL);
+
+	}
+
+	DB( g_print(" -- end getlast") );
+
+}
+
+
+static void ui_bud_manage_set(GtkWidget *widget, gpointer user_data)
+{
+struct ui_bud_manage_data *data;
+gint active;
+
+	data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(widget), GTK_TYPE_WINDOW)), "inst_data");
+
+	DB( g_print("\n[ui-budget] set\n") );
+
+	active = (data->cat->flags & GF_CUSTOM) ? 1 : 0;
+
+	g_signal_handlers_block_by_func (data->CM_type[0], G_CALLBACK (ui_bud_manage_toggle), NULL);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->CM_type[active]), TRUE);
+	g_signal_handlers_unblock_by_func (data->CM_type[0], G_CALLBACK (ui_bud_manage_toggle), NULL);
+
+	for(guint i=0;i<=12;i++)
+	{
+		g_signal_handlers_block_by_func (data->spinner[i], G_CALLBACK (ui_bud_manage_cb_budget_changed), NULL);
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(data->spinner[i]), data->cat->budget[i]);
+		g_signal_handlers_unblock_by_func (data->spinner[i], G_CALLBACK (ui_bud_manage_cb_budget_changed), NULL);
+	}
+
+	g_signal_handlers_block_by_func (data->CM_force, G_CALLBACK (ui_bud_manage_cb_forcemonitor_toggled), NULL);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->CM_force), (data->cat->flags & GF_FORCED) ? 1 : 0);
+	g_signal_handlers_unblock_by_func (data->CM_force, G_CALLBACK (ui_bud_manage_cb_forcemonitor_toggled), NULL);
+
+	DB( g_print(" -- end set\n") );
 }
 
 
@@ -684,7 +915,7 @@ gint result, i;
 
 	data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(widget), GTK_TYPE_WINDOW)), "inst_data");
 
-	DB( g_print("(ui_bud_manage) clear\n") );
+	DB( g_print("\n[ui-budget] clear\n") );
 
 
 	title = _("Are you sure you want to clear input?");
@@ -692,7 +923,7 @@ gint result, i;
 	secondtext = _("If you proceed, every amount will be set to 0.");
 	
 	result = ui_dialog_msg_confirm_alert(
-			GTK_WINDOW(data->window),
+			GTK_WINDOW(data->dialog),
 			title,
 			secondtext,
 			_("_Clear")
@@ -701,166 +932,41 @@ gint result, i;
 	if( result == GTK_RESPONSE_OK )
 	{
 		//g_signal_handler_block(data->CM_type[0], data->handler_id[HID_CUSTOM]);
+
+		
+		
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->CM_type[0]), TRUE);
 		//g_signal_handler_unblock(data->CM_type[0], data->handler_id[HID_CUSTOM]);
 
 		for(i=0;i<=12;i++)
 		{
-			//g_signal_handler_block(data->spinner[i], data->spinner_hid[i]);
-
+			g_signal_handlers_block_by_func (data->spinner[i], G_CALLBACK (ui_bud_manage_cb_budget_changed), NULL);
 			gtk_spin_button_set_value(GTK_SPIN_BUTTON(data->spinner[i]), 0);
 			data->cat->budget[i] = 0;
-
-			//g_signal_handler_unblock(data->spinner[i], data->spinner_hid[i]);
+			g_signal_handlers_unblock_by_func (data->spinner[i], G_CALLBACK (ui_bud_manage_cb_budget_changed), NULL);
 		}
 
 		data->cat->flags &= ~(GF_BUDGET);	//delete flag
 		data->change++;
 		
-		gtk_widget_queue_draw (data->LV_cat);
+		hbtk_listview_redraw_selected_row (GTK_TREE_VIEW(data->LV_cat));
+		ui_bud_manage_compute_total(data->dialog, NULL);
+
 	}
 		
 }
 
 
-static void ui_bud_manage_set(GtkWidget *widget, gpointer user_data)
-{
-struct ui_bud_manage_data *data;
-gint active;
-gint i;
-
-	data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(widget), GTK_TYPE_WINDOW)), "inst_data");
-
-	DB( g_print("(ui_bud_manage) set\n") );
-
-	active = data->cat->flags & GF_CUSTOM ? 1 : 0;
-	//data->custom = active;
-
-	//DB( g_print(" set custom to %d\n", data->custom) );
-
-	g_signal_handler_block(data->CM_type[0], data->handler_id[HID_CUSTOM]);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->CM_type[active]), TRUE);
-	g_signal_handler_unblock(data->CM_type[0], data->handler_id[HID_CUSTOM]);
-
-	for(i=0;i<=12;i++)
-	{
-		//g_signal_handler_block(data->spinner[i], data->spinner_hid[i]);
-
-		gtk_spin_button_set_value(GTK_SPIN_BUTTON(data->spinner[i]), data->cat->budget[i]);
-		//DB( g_print(" %.2f\n", data->cat->budget[i]) );
-
-		//g_signal_handler_unblock(data->spinner[i], data->spinner_hid[i]);
-	}
-
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->CM_force), (data->cat->flags & GF_FORCED) ? 1 : 0);
-
-}
-
-static gboolean ui_bud_manage_has_budget(GtkSpinButton *spinbutton, gpointer user_data)
-{
-struct ui_bud_manage_data *data;
-gint i;
-Category *item;
-gboolean retval;
-
-	data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(spinbutton), GTK_TYPE_WINDOW)), "inst_data");
-
-	DB( g_print("(ui_bud_manage) has budget\n") );
-
-	retval = FALSE;
-	
-	item = data->cat;
-
-	if( item != NULL )
-	{
-		item->flags &= ~(GF_BUDGET);	//delete flag
-		for(i=0;i<=12;i++)
-		{
-			gtk_spin_button_update(GTK_SPIN_BUTTON(data->spinner[i]));
-			if( gtk_spin_button_get_value(GTK_SPIN_BUTTON(data->spinner[i])) )
-			{
-				retval = TRUE;
-				item->flags |= GF_BUDGET;
-				break;
-			}
-		}
-		
-	}
-	return retval;
-}
-
-
-static void ui_bud_manage_getlast(struct ui_bud_manage_data *data)
-{
-gboolean budget, change;
-gint i;
-Category *item;
-gdouble oldvalue;
-	gint active;
-
-	item = data->lastcatitem;
-
-	DB( g_print("****\n(ui_bud_manage) getlast for '%s'\n", item->name ) );
-
-	if( item != NULL )
-	{
-	gushort old_flags = item->flags;
-
-		item->flags &= ~(GF_CUSTOM);
-		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->CM_type[0])) == FALSE)
-			item->flags |= GF_CUSTOM;
-
-		DB( g_print(" custom flag=%d\n", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->CM_type[1]))) );
-
-		// if any value,set the flag to visual indicator
-		budget = FALSE;
-		change = FALSE;
-		item->flags &= ~(GF_BUDGET);	//delete flag
-		for(i=0;i<=12;i++)
-		{
-			gtk_spin_button_update(GTK_SPIN_BUTTON(data->spinner[i]));
-			oldvalue = item->budget[i];
-
-			item->budget[i] = gtk_spin_button_get_value(GTK_SPIN_BUTTON(data->spinner[i]));
-
-			if( oldvalue != item->budget[i])
-				change = TRUE;
-
-			DB( g_print(" set value %d to %.2f\n", i, item->budget[i]) );
-			if(item->budget[i])
-			{
-				budget = TRUE;
-			}
-		}
-
-		item->flags &= ~(GF_FORCED);	//delete flag
-		active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->CM_force));
-		if(active == 1)
-			item->flags |= GF_FORCED;
-
-		if(budget == TRUE || active == 1)
-			item->flags |= GF_BUDGET;
-
-		// compute changes
-		if( (old_flags != item->flags) || change )
-			data->change++;
-
-		gtk_widget_queue_draw (data->LV_cat);
-
-	}
-
-}
-
-
 static void ui_bud_manage_selection_change(GtkWidget *treeview, gpointer user_data)
 {
 struct ui_bud_manage_data *data;
 GtkTreeModel *model;
 GtkTreeIter iter;
+gchar *title;
 
 	data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(treeview), GTK_TYPE_WINDOW)), "inst_data");
 
-	DB( g_print("(ui_bud_manage) changed\n") );
+	DB( g_print("\n[ui-budget] selection changed\n") );
 
 	data->cat = NULL;
 
@@ -874,6 +980,10 @@ GtkTreeIter iter;
 
 		DB( g_print(" selected %s\n", item->name) );
 
+		title = g_strdup_printf(_("Budget for %s"), item->fullname);
+		gtk_label_set_text(GTK_LABEL(data->label_budget), title);
+		g_free(title);
+		
 		if(data->lastcatitem != NULL && item != data->lastcatitem)
 		{
 			DB( g_print(" -> should do a get for last selected (%s)\n", data->lastcatitem->name) );
@@ -889,38 +999,50 @@ GtkTreeIter iter;
 	else
 	{
 		data->lastcatitem = NULL;
+		gtk_label_set_text(GTK_LABEL(data->label_budget), NULL);
 	}
 
+
 	ui_bud_manage_update(treeview, NULL);
 
+
+	DB( g_print(" -- end selection changed\n") );
+
 }
 
-static void ui_bud_manage_toggle(GtkRadioButton *radiobutton, gpointer user_data)
+static void ui_bud_manage_populate_listview(struct ui_bud_manage_data *data)
 {
-//struct ui_bud_manage_data *data;
+gint type;
 
-	//data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(radiobutton), GTK_TYPE_WINDOW)), "inst_data");
+	DB( g_print("\n[ui-budget] populate listview\n") );
+	
+	type = hbtk_radio_button_get_active(GTK_CONTAINER(data->RA_type)) == 1 ? CAT_TYPE_INCOME : CAT_TYPE_EXPENSE;
 
-	DB( g_print("(ui_bud_manage) toggle\n") );
+	ui_cat_listview_populate(data->LV_cat, type);
+	//gtk_tree_view_expand_all (GTK_TREE_VIEW(data->LV_cat));
 
-	//ui_bud_manage_get(GTK_WIDGET(radiobutton), GINT_TO_POINTER(FIELD_TYPE));
+	DB( g_print(" -- end populate listview\n") );
 
-	//data->custom ^= 1;
-	ui_bud_manage_update(GTK_WIDGET(radiobutton), NULL);
 }
 
 
-void ui_bud_manage_selection(GtkTreeSelection *treeselection, gpointer user_data)
+static void ui_bud_manage_selection(GtkTreeSelection *treeselection, gpointer user_data)
 {
 	ui_bud_manage_selection_change(GTK_WIDGET(gtk_tree_selection_get_tree_view (treeselection)), NULL);
 }
 
+static void ui_bud_manage_cb_type_changed (GtkToggleButton *button, gpointer user_data)
+{
+	ui_bud_manage_populate_listview(user_data);
+	//g_print(" toggle type=%d\n", gtk_toggle_button_get_active(button));
+}
+
 
 static gboolean ui_bud_manage_cleanup(struct ui_bud_manage_data *data, gint result)
 {
 gboolean doupdate = FALSE;
 
-	DB( g_print("(ui_bud_manage) cleanup\n") );
+	DB( g_print("\n[ui-budget] cleanup\n") );
 
 	if(data->lastcatitem != NULL)
 	{
@@ -938,21 +1060,14 @@ gboolean doupdate = FALSE;
 	return doupdate;
 }
 
-static void ui_bud_manage_populate_listview(struct ui_bud_manage_data *data)
-{
-gint type;
 
-	type = hbtk_radio_button_get_active(GTK_CONTAINER(data->RA_type)) == 1 ? CAT_TYPE_INCOME : CAT_TYPE_EXPENSE;
 
-	ui_cat_listview_populate(data->LV_cat, type);
-	gtk_tree_view_expand_all (GTK_TREE_VIEW(data->LV_cat));
-}
 
 
 static void ui_bud_manage_setup(struct ui_bud_manage_data *data)
 {
 
-	DB( g_print("(ui_bud_manage) setup\n") );
+	DB( g_print("\n[ui-budget] setup\n") );
 
 	data->tmp_list = NULL;
 	data->change = 0;
@@ -960,25 +1075,19 @@ static void ui_bud_manage_setup(struct ui_bud_manage_data *data)
 	data->lastcatitem = NULL;
 
 	ui_bud_manage_populate_listview(data);
-	
+	ui_bud_manage_compute_total(data->dialog, NULL);
+	ui_bud_manage_update(data->dialog, NULL);
+
+	DB( g_print("-- end setup\n") );
+
 }
 
 
-static void ui_bud_manage_type_changed_cb (GtkToggleButton *button, gpointer user_data)
-{
-	ui_bud_manage_populate_listview(user_data);
-	//g_print(" toggle type=%d\n", gtk_toggle_button_get_active(button));
-}
-
-
-
-
-
 GtkWidget *ui_bud_manage_dialog (void)
 {
-struct ui_bud_manage_data data;
+struct ui_bud_manage_data *data;
 GtkWidget *dialog, *content_area;
-GtkWidget *content_grid, *group_grid, *table, *scrollwin, *label;
+GtkWidget *content_grid, *group_grid, *scrollwin, *label;
 GtkWidget *treeview, *hpaned, *bbox, *vbox, *hbox;
 GtkWidget *menu, *menuitem, *widget, *image, *tbar;
 GtkToolItem *toolitem;
@@ -987,7 +1096,8 @@ guint i;
 gint w, h;
 gint crow, row;
 
-	memset(&data, 0, sizeof(struct ui_bud_manage_data));
+	data = g_malloc0(sizeof(struct ui_bud_manage_data));
+	if(!data) return NULL;
 
 	dialog = gtk_dialog_new_with_buttons (_("Manage Budget"),
 					    GTK_WINDOW(GLOBALS->mainwindow),
@@ -996,7 +1106,7 @@ gint crow, row;
 					    GTK_RESPONSE_ACCEPT,
 					    NULL);
 
-	data.window = dialog;
+	data->dialog = dialog;
 
 	gtk_window_set_icon_name(GTK_WINDOW (dialog), ICONNAME_HB_BUDGET);
 
@@ -1006,37 +1116,41 @@ gint crow, row;
 
 	
 	//store our window private data
-	g_object_set_data(G_OBJECT(dialog), "inst_data", (gpointer)&data);
-	DB( g_print("(ui_bud_manage) window=%p, inst_data=%p\n", dialog, &data) );
+	g_object_set_data(G_OBJECT(dialog), "inst_data", (gpointer)data);
+	DB( g_print("\n[ui-budget] window=%p, inst_data=%p\n", dialog, data) );
 
 	//window contents
 	content_area = gtk_dialog_get_content_area(GTK_DIALOG (dialog));	 	// return a vbox
 
-  //our table
-	table = gtk_grid_new ();
-	gtk_grid_set_row_spacing (GTK_GRID (table), SPACING_MEDIUM);
-	gtk_grid_set_column_spacing (GTK_GRID (table), SPACING_MEDIUM);
-	g_object_set(table, "margin", SPACING_MEDIUM, NULL);
-	gtk_box_pack_start (GTK_BOX (content_area), table, TRUE, TRUE, 0);
+	hpaned = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
+	gtk_container_set_border_width (GTK_CONTAINER(hpaned), SPACING_MEDIUM);
+	gtk_box_pack_start (GTK_BOX (content_area), hpaned, TRUE, TRUE, 0);
+
+	/* left area */
+	//list
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+	gtk_widget_set_margin_end(vbox, SPACING_SMALL);
+	gtk_paned_pack1 (GTK_PANED(hpaned), vbox, FALSE, FALSE);
+
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_widget_set_margin_bottom(hbox, SPACING_MEDIUM);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-	crow = 0;
 	bbox = hbtk_radio_button_new(CYA_CAT_TYPE, TRUE);
-	data.RA_type = bbox;
+	data->RA_type = bbox;
 	gtk_widget_set_halign (bbox, GTK_ALIGN_CENTER);
-	gtk_grid_attach (GTK_GRID (table), bbox, 0, crow, 1, 1);
-
-	hbtk_radio_button_connect (GTK_CONTAINER(bbox), "toggled", G_CALLBACK (ui_bud_manage_type_changed_cb), &data);
+	gtk_box_pack_start (GTK_BOX (hbox), bbox, TRUE, TRUE, 0);
 
 	menu = gtk_menu_new ();
 	gtk_widget_set_halign (menu, GTK_ALIGN_END);
 
 	menuitem = gtk_menu_item_new_with_mnemonic (_("_Import CSV"));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	g_signal_connect (G_OBJECT (menuitem), "activate", G_CALLBACK (ui_bud_manage_load_csv), &data);
+	g_signal_connect (G_OBJECT (menuitem), "activate", G_CALLBACK (ui_bud_manage_load_csv), data);
 
 	menuitem = gtk_menu_item_new_with_mnemonic (_("E_xport CSV"));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	g_signal_connect (G_OBJECT (menuitem), "activate", G_CALLBACK (ui_bud_manage_save_csv), &data);
+	g_signal_connect (G_OBJECT (menuitem), "activate", G_CALLBACK (ui_bud_manage_save_csv), data);
 	
 	gtk_widget_show_all (menu);
 	
@@ -1051,27 +1165,15 @@ gint crow, row;
 
 	gtk_widget_set_hexpand (widget, FALSE);
 	gtk_widget_set_halign (widget, GTK_ALIGN_END);
-	gtk_grid_attach (GTK_GRID (table), widget, 0, crow++, 1, 1);
+	gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 0);
 
 
-
-	crow++;
-	hpaned = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
-	//gtk_container_set_border_width (GTK_CONTAINER(hpaned), SPACING_MEDIUM);
-	gtk_grid_attach (GTK_GRID (table), hpaned, 0, crow++, 1, 1);
-
-	/* left area */
-	//list
-	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-	gtk_widget_set_margin_right(vbox, SPACING_SMALL);
-	gtk_paned_pack1 (GTK_PANED(hpaned), vbox, FALSE, FALSE);
-	
 	scrollwin = gtk_scrolled_window_new(NULL,NULL);
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrollwin), GTK_SHADOW_ETCHED_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_min_content_height(GTK_SCROLLED_WINDOW(scrollwin), HB_MINHEIGHT_LIST);
  	treeview = (GtkWidget *)ui_bud_listview_new();
- 	data.LV_cat = treeview;
-	gtk_widget_set_size_request(treeview, HB_MINWIDTH_LIST, -1);
+ 	data->LV_cat = treeview;
 	gtk_container_add(GTK_CONTAINER(scrollwin), treeview);
 	gtk_widget_set_hexpand (scrollwin, TRUE);
 	gtk_widget_set_vexpand (scrollwin, TRUE);
@@ -1096,11 +1198,11 @@ gint crow, row;
 	gtk_toolbar_insert(GTK_TOOLBAR(tbar), GTK_TOOL_ITEM(toolitem), -1);
 	
 		widget = make_image_button(ICONNAME_HB_BUTTON_EXPAND, _("Expand all"));
-		data.BT_expand = widget;
+		data->BT_expand = widget;
 		gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 0);
 
 		widget = make_image_button(ICONNAME_HB_BUTTON_COLLAPSE, _("Collapse all"));
-		data.BT_collapse = widget;
+		data->BT_collapse = widget;
 		gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 0);
 
 	
@@ -1109,41 +1211,74 @@ gint crow, row;
 	gtk_grid_set_row_spacing (GTK_GRID (content_grid), SPACING_LARGE);
 	gtk_orientable_set_orientation(GTK_ORIENTABLE(content_grid), GTK_ORIENTATION_VERTICAL);
 	//gtk_container_set_border_width (GTK_CONTAINER(content_grid), SPACING_MEDIUM);
-	gtk_widget_set_margin_left(content_grid, SPACING_SMALL);
+	gtk_widget_set_margin_start(content_grid, SPACING_SMALL);
 	gtk_paned_pack2 (GTK_PANED(hpaned), content_grid, FALSE, FALSE);
 
 	crow = 0;
 
-	
 	// group :: General
     group_grid = gtk_grid_new ();
 	gtk_grid_set_row_spacing (GTK_GRID (group_grid), SPACING_SMALL);
 	gtk_grid_set_column_spacing (GTK_GRID (group_grid), SPACING_MEDIUM);
 	gtk_grid_attach (GTK_GRID (content_grid), group_grid, 0, crow++, 1, 1);
+
+	gtk_widget_set_halign(group_grid, GTK_ALIGN_END);
 	
-	label = make_label_group(_("Budget for each month"));
-	data.label_budget = label;
-	gtk_grid_attach (GTK_GRID (group_grid), label, 0, 0, 3, 1);
+	//label = make_label_group(_("Budget total"));
+	//gtk_grid_attach (GTK_GRID (group_grid), label, 0, 0, 3, 1);
+
+	row = 1;
+	label = gtk_label_new(_("Expense:"));
+ 	gtk_grid_attach (GTK_GRID (group_grid), label, 1, row, 1, 1);
+
+	label = gtk_label_new(NULL);
+	data->TX_totexp = label;
+ 	gtk_grid_attach (GTK_GRID (group_grid), label, 2, row, 1, 1);
+
+	label = gtk_label_new(_("Income:"));
+ 	gtk_grid_attach (GTK_GRID (group_grid), label, 3, row, 1, 1);
+
+	label = gtk_label_new(NULL);
+	data->TX_totinc = label;
+ 	gtk_grid_attach (GTK_GRID (group_grid), label, 4, row, 1, 1);
+
+	label = gtk_label_new(_("Balance:"));
+ 	gtk_grid_attach (GTK_GRID (group_grid), label, 5, row, 1, 1);
+
+	label = gtk_label_new(NULL);
+	data->TX_totbal = label;
+ 	gtk_grid_attach (GTK_GRID (group_grid), label, 6, row, 1, 1);
+	
+	// group :: General
+	crow++;
+	group_grid = gtk_grid_new ();
+	gtk_grid_set_row_spacing (GTK_GRID (group_grid), SPACING_SMALL);
+	gtk_grid_set_column_spacing (GTK_GRID (group_grid), SPACING_MEDIUM);
+	gtk_grid_attach (GTK_GRID (content_grid), group_grid, 0, crow++, 1, 1);
+	
+	label = make_label_group(NULL);
+	data->label_budget = label;
+	gtk_grid_attach (GTK_GRID (group_grid), label, 0, 0, 5, 1);
 
 	fchain = NULL;
 	
 	row = 1;
-	widget = gtk_radio_button_new_with_label (NULL, _("is the same"));
-	data.CM_type[0] = widget;
+	widget = gtk_radio_button_new_with_label (NULL, _("is the same each month"));
+	data->CM_type[0] = widget;
 	gtk_widget_set_hexpand (widget, TRUE);
  	gtk_grid_attach (GTK_GRID (group_grid), widget, 1, row, 4, 1);
 	fchain = g_list_append(fchain, widget);
 
 	row++;
 	widget = make_amount(label);
-	data.spinner[0] = widget;
+	data->spinner[0] = widget;
 	gtk_grid_attach (GTK_GRID (group_grid), widget, 2, row, 1, 1);
 	fchain = g_list_append(fchain, widget);
 
-	g_signal_connect (G_OBJECT (data.spinner[0]), "value-changed", G_CALLBACK (ui_bud_manage_has_budget), NULL);
+	g_signal_connect (G_OBJECT (data->spinner[0]), "value-changed", G_CALLBACK (ui_bud_manage_cb_budget_changed), NULL);
 	
 	widget =  gtk_button_new_with_mnemonic (_("_Clear input"));
-	data.BT_clear = widget;
+	data->BT_clear = widget;
 	gtk_widget_set_hexpand (widget, TRUE);
 	gtk_widget_set_halign(widget, GTK_ALIGN_START);
 	gtk_grid_attach (GTK_GRID (group_grid), widget, 4, row, 1, 1);
@@ -1157,8 +1292,8 @@ gint crow, row;
 	*/
 
  	row++;
-    widget = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (data.CM_type[0]), _("is different"));
-	data.CM_type[1] = widget;
+    widget = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (data->CM_type[0]), _("is different per month"));
+	data->CM_type[1] = widget;
 	gtk_widget_set_hexpand (widget, TRUE);
 	gtk_grid_attach (GTK_GRID (group_grid), widget, 1, row, 4, 1);
 	fchain = g_list_append(fchain, widget);
@@ -1173,18 +1308,18 @@ gint crow, row;
 
 		//#1826659 budget dialog month widget label decay by one
 		label = make_label_widget(_(CYA_ABMONTHS[i+1]));
-		data.label[i+1] = label;
+		data->label[i+1] = label;
 		gtk_grid_attach (GTK_GRID (group_grid), label, l, t, 1, 1);
 
 		widget = make_amount(label);
-		data.spinner[i+1] = widget;
+		data->spinner[i+1] = widget;
 		fchain = g_list_append(fchain, widget);
 		gtk_widget_set_hexpand (widget, TRUE);
 		gtk_grid_attach (GTK_GRID (group_grid), widget, l+1, t, 1, 1);
 
-		g_signal_connect (G_OBJECT (data.spinner[i+1]), "value-changed", G_CALLBACK (ui_bud_manage_has_budget), NULL);
+		g_signal_connect (G_OBJECT (data->spinner[i+1]), "value-changed", G_CALLBACK (ui_bud_manage_cb_budget_changed), NULL);
 		
-		//DB( g_print("(ui_bud_manage) %s, col=%d, row=%d", CYA_ABMONTHS[i], col, row) );
+		//DB( g_print("\n[ui-budget] %s, col=%d, row=%d", CYA_ABMONTHS[i], col, row) );
 	}
 
 	gtk_container_set_focus_chain(GTK_CONTAINER(group_grid), fchain);
@@ -1197,12 +1332,12 @@ gint crow, row;
 	gtk_grid_attach (GTK_GRID (content_grid), group_grid, 0, crow++, 1, 1);
 
 	label = make_label_group(_("Options"));
-	data.label_options = label;
+	data->label_options = label;
 	gtk_grid_attach (GTK_GRID (group_grid), label, 0, 0, 3, 1);
 
 	row = 1;
 	widget = gtk_check_button_new_with_mnemonic (_("_Force monitoring this category"));
-	data.CM_force = widget;
+	data->CM_force = widget;
 	gtk_widget_set_hexpand (widget, TRUE);
 	gtk_grid_attach (GTK_GRID (group_grid), widget, 1, row, 4, 1);
 
@@ -1211,25 +1346,25 @@ gint crow, row;
     g_signal_connect (dialog, "destroy",
 			G_CALLBACK (gtk_widget_destroyed), &dialog);
 
+	hbtk_radio_button_connect (GTK_CONTAINER(data->RA_type), "toggled", G_CALLBACK (ui_bud_manage_cb_type_changed), data);
 
-	g_signal_connect (gtk_tree_view_get_selection(GTK_TREE_VIEW(data.LV_cat)), "changed", G_CALLBACK (ui_bud_manage_selection), NULL);
-	//g_signal_connect (GTK_TREE_VIEW(data.LV_cat), "row-activated", G_CALLBACK (ui_bud_manage_onRowActivated), NULL);
+	g_signal_connect (gtk_tree_view_get_selection(GTK_TREE_VIEW(data->LV_cat)), "changed", G_CALLBACK (ui_bud_manage_selection), NULL);
+	//g_signal_connect (GTK_TREE_VIEW(data->LV_cat), "row-activated", G_CALLBACK (ui_bud_manage_onRowActivated), NULL);
 
-	g_signal_connect (G_OBJECT (data.BT_expand), "clicked", G_CALLBACK (ui_bud_manage_expand_all), NULL);
-	g_signal_connect (G_OBJECT (data.BT_collapse), "clicked", G_CALLBACK (ui_bud_manage_collapse_all), NULL);
+	g_signal_connect (G_OBJECT (data->BT_expand), "clicked", G_CALLBACK (ui_bud_manage_expand_all), NULL);
+	g_signal_connect (G_OBJECT (data->BT_collapse), "clicked", G_CALLBACK (ui_bud_manage_collapse_all), NULL);
 
-	data.handler_id[HID_CUSTOM] = g_signal_connect (data.CM_type[0], "toggled", G_CALLBACK (ui_bud_manage_toggle), NULL);
+	g_signal_connect (data->CM_type[0], "toggled", G_CALLBACK (ui_bud_manage_toggle), NULL);
 
-	g_signal_connect (G_OBJECT (data.BT_clear), "clicked", G_CALLBACK (ui_bud_manage_clear), NULL);
+	g_signal_connect (G_OBJECT (data->BT_clear), "clicked", G_CALLBACK (ui_bud_manage_clear), NULL);
 
-	//data.custom = FALSE;
-	//gtk_widget_set_sensitive(data.table, FALSE);
+	g_signal_connect (data->CM_force, "toggled", G_CALLBACK (ui_bud_manage_cb_forcemonitor_toggled), NULL);
+	
+	//data->custom = FALSE;
+	//gtk_widget_set_sensitive(data->table, FALSE);
 
 	//setup, init and show window
-	ui_bud_manage_setup(&data);
-	ui_bud_manage_update(dialog, NULL);
-
-
+	ui_bud_manage_setup(data);
 
 	gtk_widget_show_all (dialog);
 
@@ -1246,9 +1381,11 @@ gint crow, row;
     }
 
 	// cleanup and destroy
-	ui_bud_manage_cleanup(&data, result);
+	ui_bud_manage_cleanup(data, result);
 	gtk_widget_destroy (dialog);
 
+	g_free(data);
+	
 	return NULL;
 }
 
