@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2019 Maxime DOYEN
+ *  Copyright (C) 1995-2020 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -334,7 +334,7 @@ gint count;
 
 	data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(widget), GTK_TYPE_WINDOW)), "inst_data");
 
-	amount= g_strtod(gtk_entry_get_text(GTK_ENTRY(data->ST_amount)), NULL);
+	amount = g_strtod(gtk_entry_get_text(GTK_ENTRY(data->ST_amount)), NULL);
 
 	tmpval = hb_amount_round(amount, 2) != 0.0 ? TRUE : FALSE;
 	gtk_widget_set_sensitive (data->BT_apply, tmpval);
@@ -405,12 +405,15 @@ struct ui_split_dialog_data *data;
 	g_signal_handler_block(data->ST_amount, data->hid_amt);
 
 	ui_cat_comboboxentry_set_active(GTK_COMBO_BOX(data->PO_cat), 0);
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(data->ST_amount), 0.0);
+	if( data->mode == SPLIT_MODE_EMPTY )
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(data->ST_amount), 0.0);
 	gtk_entry_set_text(GTK_ENTRY(data->ST_memo), "");
 
 	g_signal_handler_unblock(data->ST_amount, data->hid_amt);
 	g_signal_handler_unblock(data->PO_cat, data->hid_cat);
 
+	gtk_widget_grab_focus(data->ST_amount);
+	
 	data->isedited = FALSE;
 }
 
@@ -496,11 +499,11 @@ GtkTreeIter			 iter;
 }
 
 
-static void ui_split_dialog_removeall_cb(GtkWidget *widget, gpointer user_data)
+static void ui_split_dialog_deleteall_cb(GtkWidget *widget, gpointer user_data)
 {
 struct ui_split_dialog_data *data;
 
-	DB( g_print("\n[ui_split_dialog] removeall_cb\n") );
+	DB( g_print("\n[ui_split_dialog] deleteall_cb\n") );
 
 	data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(widget), GTK_TYPE_WINDOW)), "inst_data");
 
@@ -513,14 +516,14 @@ struct ui_split_dialog_data *data;
 }
 
 
-static void ui_split_dialog_remove_cb(GtkWidget *widget, gpointer user_data)
+static void ui_split_dialog_delete_cb(GtkWidget *widget, gpointer user_data)
 {
 struct ui_split_dialog_data *data;
 GtkTreeSelection *selection;
 GtkTreeModel		 *model;
 GtkTreeIter			 iter;
 
-	DB( g_print("\n[ui_split_dialog] remove_cb\n") );
+	DB( g_print("\n[ui_split_dialog] delete_cb\n") );
 
 	data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(widget), GTK_TYPE_WINDOW)), "inst_data");
 
@@ -531,8 +534,8 @@ GtkTreeIter			 iter;
 
 		gtk_tree_model_get(model, &iter, 0, &split, -1);
 		//todo: not implemented yet
-		da_splits_remove(data->tmp_splits, split);
 		gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
+		da_splits_delete(data->tmp_splits, split);
 	}
 	
 	ui_split_dialog_compute (widget, data);
@@ -561,6 +564,10 @@ gdouble amount;
 		amount = gtk_spin_button_get_value(GTK_SPIN_BUTTON(data->ST_amount));
 		if(amount)
 		{
+			//#1848604 force sign according txn type
+			if( hb_amount_type_match(amount, data->txntype) == FALSE)
+				amount *= -1;
+
 			split->amount = amount;
 			split->kcat = ui_cat_comboboxentry_get_key_add_new(GTK_COMBO_BOX(data->PO_cat));
 			split->memo = g_strdup((gchar *)gtk_entry_get_text(GTK_ENTRY(data->ST_memo)));
@@ -576,7 +583,6 @@ gdouble amount;
 				-1);
 
 			ui_split_dialog_compute (widget, data);
-			
 		}
 		else
 		{
@@ -625,7 +631,7 @@ void ui_split_dialog_compute(GtkWidget *widget, gpointer user_data)
 {
 struct ui_split_dialog_data *data;
 gint i, count, nbvalid;
-gchar buf[48];
+gchar buf[48], *msg;
 gboolean sensitive;
 GtkTreeModel *model;
 GtkTreeIter	iter;
@@ -663,19 +669,24 @@ gboolean valid;
 	data->remsplit = data->amount - data->sumsplit;
 
 
-	//validation: 2 split min
+	//validation: 2 split min, if 0 split
+	//Accept button is not disabled to enable empty the splits
 	sensitive = FALSE;
 	if( (count == 0) || nbvalid >= 2 )
 		sensitive = TRUE;
-	gtk_dialog_set_response_sensitive(GTK_DIALOG(data->dialog), GTK_RESPONSE_ACCEPT, sensitive);
 
-	if( hb_amount_round(data->amount, 2) != 0.0 )
+	if( data->mode == SPLIT_MODE_AMOUNT )
 	{
 		if(!data->remsplit)
 			g_sprintf(buf, "----");
 		else
+		{
 			g_snprintf(buf, 48, "%.2f", data->remsplit);
-
+			//#1845841 bring back checkpoint with initial amount + init remainder
+			sensitive = (count > 1) ? FALSE : sensitive;
+			gtk_spin_button_set_value(GTK_SPIN_BUTTON(data->ST_amount), data->remsplit);
+		}
+				
 		gtk_label_set_label(GTK_LABEL(data->LB_remain), buf);
 
 		g_snprintf(buf, 48, "%.2f", data->amount);
@@ -684,7 +695,17 @@ gboolean valid;
 
 	g_snprintf(buf, 48, "%.2f", data->sumsplit);
 	gtk_label_set_text(GTK_LABEL(data->LB_sumsplit), buf);
-	
+
+	//if split sum sign do not match
+	msg = "";
+	if( hb_amount_type_match(data->sumsplit, data->txntype) == FALSE )
+	{
+		sensitive = FALSE;
+		msg = _("Warning: sum of splits and transaction type don't match");
+	}
+	gtk_label_set_text(GTK_LABEL(data->LB_msg), msg);
+
+	gtk_dialog_set_response_sensitive(GTK_DIALOG(data->dialog), GTK_RESPONSE_ACCEPT, sensitive);
 }
 
 
@@ -709,7 +730,7 @@ guint count;
 }
 
 
-GtkWidget *ui_split_dialog (GtkWidget *parent, GPtrArray **src_splits, gdouble amount, void (update_callbackFunction(GtkWidget*, gdouble)))
+GtkWidget *ui_split_dialog (GtkWidget *parent, GPtrArray **src_splits, gint txntype, gdouble amount, void (update_callbackFunction(GtkWidget*, gdouble)))
 {
 struct ui_split_dialog_data *data;
 GtkWidget *dialog, *content, *table, *box, *scrollwin;
@@ -719,6 +740,7 @@ gint row;
 	DB( g_print("\n[ui_split_dialog] new\n") );
 
 	data = g_malloc0(sizeof(struct ui_split_dialog_data));
+	if(!data) return NULL;
 
 	dialog = gtk_dialog_new_with_buttons (_("Transaction splits"),
 					    GTK_WINDOW(parent),
@@ -736,15 +758,16 @@ gint row;
 
 	data->dialog = dialog;
 
-	//gtk_dialog_add_button(GTK_DIALOG(dialog), _("_Remove"), GTK_RESPONSE_SPLIT_REM);
 	gtk_dialog_add_button(GTK_DIALOG(dialog), _("_OK"), GTK_RESPONSE_ACCEPT);
 	
 	//todo: init should move 
 	//clone splits or create new
 	data->src_splits = *src_splits;
+	data->txntype = txntype;
+	data->mode = (hb_amount_round(amount, 2) != 0.0) ? SPLIT_MODE_AMOUNT : SPLIT_MODE_EMPTY;
 	data->amount = amount;
 	data->sumsplit = amount;
-	
+
 	if( *src_splits != NULL )
 		data->tmp_splits = da_splits_clone(*src_splits); 
 	else
@@ -775,11 +798,11 @@ gint row;
 	gtk_widget_set_valign (box, GTK_ALIGN_CENTER);
 	gtk_grid_attach (GTK_GRID (table), box, 4, row, 1, 1);
 
-		widget = make_image_button(ICONNAME_LIST_REMOVE_ALL, _("Remove all"));
+		widget = make_image_button(ICONNAME_LIST_DELETE_ALL, _("Delete all"));
 		data->BT_remall = widget;
 		gtk_box_pack_end (GTK_BOX (box), widget, FALSE, FALSE, 0);
 
-		widget = make_image_button(ICONNAME_LIST_REMOVE, _("Remove"));
+		widget = make_image_button(ICONNAME_LIST_DELETE, _("Delete"));
 		data->BT_rem = widget;
 		gtk_box_pack_end (GTK_BOX(box), widget, FALSE, FALSE, 0);
 
@@ -834,7 +857,7 @@ gint row;
 		gtk_box_pack_start (GTK_BOX(box), widget, FALSE, FALSE, 0);
 
 
-	if( hb_amount_round(data->amount, 2) != 0.0 )
+	if( data->mode == SPLIT_MODE_AMOUNT )
 	{
 		row++;
 		label = gtk_label_new(_("Transaction amount:"));
@@ -869,6 +892,11 @@ gint row;
 	data->LB_sumsplit = widget;
 	gtk_grid_attach (GTK_GRID (table), widget, 2, row, 1, 1);
 
+	row++;
+	label = gtk_label_new(NULL);
+	gtk_widget_set_halign (label, GTK_ALIGN_END);
+	data->LB_msg = label;
+	gtk_grid_attach (GTK_GRID (table), label, 0, row, 4, 1);
 
 	//connect all our signals
 	g_signal_connect (gtk_tree_view_get_selection(GTK_TREE_VIEW(data->LV_split)), "changed", G_CALLBACK (ui_split_selection), data);
@@ -885,8 +913,8 @@ gint row;
 
 	
 	g_signal_connect (data->BT_edit  , "clicked", G_CALLBACK (ui_split_dialog_edit_start), NULL);
-	g_signal_connect (data->BT_rem   , "clicked", G_CALLBACK (ui_split_dialog_remove_cb), NULL);
-	g_signal_connect (data->BT_remall, "clicked", G_CALLBACK (ui_split_dialog_removeall_cb), NULL);
+	g_signal_connect (data->BT_rem   , "clicked", G_CALLBACK (ui_split_dialog_delete_cb), NULL);
+	g_signal_connect (data->BT_remall, "clicked", G_CALLBACK (ui_split_dialog_deleteall_cb), NULL);
 
 	g_signal_connect (data->BT_add   , "clicked", G_CALLBACK (ui_split_dialog_add_cb), NULL);
 	g_signal_connect (data->BT_apply , "clicked", G_CALLBACK (ui_split_dialog_apply_cb), NULL);
@@ -915,7 +943,7 @@ gint row;
 		}
 		else
 		{
-			//remove split and revert back original amount
+			//delete split and revert back original amount
 			da_split_destroy(*src_splits);
 			*src_splits = NULL;
 			update_callbackFunction(parent, data->amount);
@@ -952,6 +980,7 @@ gint row;
 	gtk_widget_destroy (dialog);
 	
 	da_split_destroy (data->tmp_splits);
+
 	g_free(data);
 
 	return NULL;
