@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2019 Maxime DOYEN
+ *  Copyright (C) 1995-2020 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -29,6 +29,11 @@ struct _hbtk_kv_data {
 	const gchar		*name;
 };
 
+struct _hbtk_kiv_data {
+	guint32			key;
+	const gchar		*iconname;
+	const gchar		*name;
+};
 
 
 #define HBTK_IS_SEPARATOR -66
@@ -48,10 +53,20 @@ GtkWidget *make_label_widget(gchar *str);
 GtkWidget *make_text(gfloat xalign);
 GtkWidget *make_search(void);
 GtkWidget *make_string(GtkWidget *label);
+
+GtkWidget *hbtk_menubar_add_menu(GtkWidget *menubar, gchar *label, GtkWidget **menuitem_ptr);
+GtkWidget *hbtk_menu_add_menuitem(GtkWidget *menu, gchar *label);
+
+GtkWidget *hbtk_toolbar_add_toolbutton(GtkToolbar *toolbar, gchar *icon_name, gchar *tooltip_text);
+
+GtkWidget *make_image_toggle_button(gchar *icon_name, gchar *tooltip_text);
 GtkWidget *make_image_button(gchar *icon_name, gchar *tooltip_text);
 
 GtkWidget *make_memo_entry(GtkWidget *label);
 GtkWidget *make_string_maxlength(GtkWidget *label, guint max_length);
+
+GtkWidget *make_entry_numeric(GtkWidget *label, gint min, gint max);
+
 GtkWidget *make_amount(GtkWidget *label);
 GtkWidget *make_exchange_rate(GtkWidget *label);
 GtkWidget *make_numeric(GtkWidget *label, gdouble min, gdouble max);
@@ -65,6 +80,7 @@ GtkWidget *create_popover (GtkWidget *parent, GtkWidget *child, GtkPositionType
 
 void ui_label_set_integer(GtkLabel *label, gint value);
 
+void hbtk_listview_redraw_selected_row(GtkTreeView *treeview);
 
 gchar *hbtk_get_label(HbKvData *kvdata, guint32 key);
 guint32 hbtk_combo_box_get_active_id (GtkComboBoxText *combobox);
@@ -87,8 +103,8 @@ void gimp_label_set_attributes (GtkLabel *label, ...);
 
 void hb_widget_visible(GtkWidget *widget, gboolean visible);
 void hbtk_entry_tag_name_append(GtkEntry *entry, gchar *tagname);
-void ui_gtk_entry_set_text(GtkWidget *widget, gchar *text);
-void ui_gtk_entry_replace_text(GtkWidget *widget, gchar **storage);
+void hbtk_entry_set_text(GtkEntry *entry, gchar *text);
+void hbtk_entry_replace_text(GtkEntry *entry, gchar **storage);
 
 /*
 guint make_popaccount_populate(GtkComboBox *combobox, GList *srclist);
@@ -104,9 +120,10 @@ guint make_popcategory_populate(GtkComboBox *combobox, GList *srclist);
 GtkWidget *make_popcategory(GtkWidget *label);
 */
 
+const gchar *get_paymode_icon_name(guint32 key);
 
-gchar *get_paymode_icon_name(gint index);
-
+guint32 paymode_combo_box_get_active (GtkComboBox *combo_box);
+void paymode_combo_box_set_active (GtkComboBox *combo_box, guint32 active_key);
 
 GtkWidget *make_paymode(GtkWidget *label);
 GtkWidget *make_paymode_nointxfer(GtkWidget *label);
